# -*- coding: utf-8 -*-
from abjad.tools.durationtools import Duration
from abjad.tools.rhythmtreetools import RhythmTreeContainer, RhythmTreeLeaf


def test_rhythmtreetools_RhythmTreeNode_duration_01():

    tree = RhythmTreeContainer(preprolated_duration=1, children=[
        RhythmTreeLeaf(preprolated_duration=1),
        RhythmTreeContainer(preprolated_duration=2, children=[
            RhythmTreeLeaf(preprolated_duration=3),
            RhythmTreeLeaf(preprolated_duration=2)
        ]),
        RhythmTreeLeaf(preprolated_duration=2)
    ])

    assert tree.duration == Duration(1)
    assert tree[0].duration == Duration(1, 5)
    assert tree[1].duration == Duration(2, 5)
    assert tree[1][0].duration == Duration(6, 25)
    assert tree[1][1].duration == Duration(4, 25)
    assert tree[2].duration == Duration(2, 5)

    tree[1].append(tree.pop())

    assert tree.duration == Duration(1)
    assert tree[0].duration == Duration(1, 3)
    assert tree[1].duration == Duration(2, 3)
    assert tree[1][0].duration == Duration(2, 7)
    assert tree[1][1].duration == Duration(4, 21)
    assert tree[1][2].duration == Duration(4, 21)

    tree.preprolated_duration = 19

    assert tree.duration == Duration(19)
    assert tree[0].duration == Duration(19, 3)
    assert tree[1].duration == Duration(38, 3)
    assert tree[1][0].duration == Duration(38, 7)
    assert tree[1][1].duration == Duration(76, 21)
    assert tree[1][2].duration == Duration(76, 21)



# -*- coding: utf-8 -*-
import copy
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import scoretools
from abjad.tools.quantizationtools.QTarget import QTarget
from abjad.tools.topleveltools import attach


class MeasurewiseQTarget(QTarget):
    r'''A measure-wise quantization target.

    Not composer-safe.

    Used internally by ``Quantizer``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE METHODS ###

    def _notate(
        self,
        attach_tempos=True,
        attack_point_optimizer=None,
        grace_handler=None,
        ):
        voice = scoretools.Voice()

        # generate the first
        q_target_measure = self.items[0]
        measure = scoretools.Measure(q_target_measure.time_signature)
        for beat in q_target_measure.beats:
            measure.extend(beat.q_grid(beat.beatspan))
        if attach_tempos:
            tempo = copy.copy(q_target_measure.tempo)
            attach(tempo, measure)
        voice.append(measure)

        # generate the rest pairwise, comparing tempi
        pairs = sequencetools.Sequence(self.items).nwise()
        for q_target_measure_one, q_target_measure_two in pairs:
            measure = scoretools.Measure(q_target_measure_two.time_signature)
            for beat in q_target_measure_two.beats:
                measure.extend(beat.q_grid(beat.beatspan))
            if ((q_target_measure_two.tempo != q_target_measure_one.tempo) and
                attach_tempos):
                tempo = copy.copy(q_target_measure_two.tempo)
                attach(tempo, measure)
            voice.append(measure)

        # apply logical ties, pitches, grace containers
        self._notate_leaves(
            grace_handler=grace_handler,
            voice=voice,
            )

        # partition logical ties in each measure
        for measure in voice:
            attack_point_optimizer(measure)

        return voice

    ### PUBLIC PROPERTIES ###

    @property
    def beats(self):
        r'''Beats of measurewise q-target.

        Returns tuple.
        '''
        return tuple([beat for item in self.items for beat in item.beats])

    @property
    def item_class(self):
        r'''Item class of measurewise q-target.

        Returns q-target measure class.
        '''
        from abjad.tools import quantizationtools
        return quantizationtools.QTargetMeasure



# -*- coding: utf-8 -*-
import collections
from abjad.tools import indicatortools
from abjad.tools import instrumenttools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools.abctools.AbjadObject import AbjadObject


class GroupedStavesScoreTemplate(AbjadObject):
    r'''Grouped staves score template.

    ::

        >>> template_class = templatetools.GroupedStavesScoreTemplate
        >>> template = template_class(staff_count=4)

    '''

    ### INITIALIZER ###

    def __init__(self, staff_count=2):
        self.context_name_abbreviations = collections.OrderedDict()
        self.staff_count = staff_count

    ### SPECIAL METHODS ###

    def __call__(self):
        '''Calls score template.

        ::

            >>> score = template()

        ::

            >>> print(format(score))
            \context Score = "Grouped Staves Score" <<
                \context StaffGroup = "Grouped Staves Staff Group" <<
                    \context Staff = "Staff 1" {
                        \context Voice = "Voice 1" {
                        }
                    }
                    \context Staff = "Staff 2" {
                        \context Voice = "Voice 2" {
                        }
                    }
                    \context Staff = "Staff 3" {
                        \context Voice = "Voice 3" {
                        }
                    }
                    \context Staff = "Staff 4" {
                        \context Voice = "Voice 4" {
                        }
                    }
                >>
            >>

        Returns score.
        '''
        staves = []
        for index in range(self.staff_count):
            number = index + 1
            voice = scoretools.Voice(name='Voice {}'.format(number))
            staff = scoretools.Staff([voice], name='Staff {}'.format(number))
            staves.append(staff)
            self.context_name_abbreviations['v{}'.format(number)] = voice.name
        grouped_rhythmic_staves_staff_group = scoretools.StaffGroup(
            staves,
            name='Grouped Staves Staff Group',
            )
        grouped_rhythmic_staves_score = scoretools.Score(
            [grouped_rhythmic_staves_staff_group],
            name='Grouped Staves Score',
            )
        return grouped_rhythmic_staves_score



# -*- coding: utf-8 -*-
import platform
import unittest
import sys
from abjad.tools import abjadbooktools
from abjad.tools import stringtools
from abjad.tools import systemtools


@unittest.skipIf(
    platform.python_implementation() != 'CPython',
    'Only for CPython.',
    )
class SphinxDocumentHandlerTests(unittest.TestCase):

    class Namespace(object):
        pass

    source_one = r'''
    ..  abjad::
        :hide:

        from abjad import *

    ::

        >>> string = 'Hello, world!'

    ..  import:: abjad.tools.abjadbooktools:example_function

    ::

        >>> print(string)
        Hello, world!

    ..  abjad::

        example_function(23)
    '''
    source_one = stringtools.normalize(source_one)

    def setUp(self):
        app = self.Namespace()
        config = self.Namespace()
        self.app = app
        self.app.config = config
        self.app.config.abjadbook_ignored_documents = ()

    def test_collect_abjad_input_blocks_01(self):
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(self.source_one)
        blocks = handler.collect_abjad_input_blocks(document)
        assert len(blocks) == 3
        nodes = tuple(blocks.keys())
        assert nodes[0] is document[0]
        assert nodes[1] is document[2]
        assert nodes[2] is document[4]

    def test_collect_python_literal_blocks_01(self):
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(self.source_one)
        blocks = handler.collect_python_literal_blocks(
            document,
            renderable_only=False,
            )
        assert len(blocks) == 2
        nodes = tuple(blocks.keys())
        assert nodes[0] is document[1]
        assert nodes[1] is document[3]

    def test_collect_python_literal_blocks_02(self):
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(self.source_one)
        blocks = handler.collect_python_literal_blocks(document)
        assert len(blocks) == 0

    def test_on_doctree_read_01(self):
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(self.source_one)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            r"""
            <document source="test">
                <literal_block xml:space="preserve">
                    >>> string = 'Hello, world!'
                <literal_block xml:space="preserve">
                    def example_function(argument):
                        r'''This is a multiline docstring.

                        This is the third line of the docstring.
                        '''
                        # This is a comment.
                        print('Entering example function.')
                        try:
                            argument = argument + 1
                        except TypeError:
                            print('Wrong type!')
                        print(argument)
                        print('Leaving example function.')
                <literal_block xml:space="preserve">
                    >>> print(string)
                    Hello, world!
                <literal_block xml:space="preserve">
                    >>> example_function(23)
                    Entering example function.
                    24
                    Leaving example function.
            """)
        assert actual == target

    @unittest.skipIf(
        sys.version[0] != '3',
        'Only for Python 3',
        )
    def test_on_doctree_read_02(self):
        source = r'''
        ..  abjad::

            staff = Staff("c'1 g'1")
            for note in staff:
                show(note)

            len(staff)
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            r"""
            <document source="test">
                <literal_block xml:space="preserve">
                    >>> staff = Staff("c'1 g'1")
                    >>> for note in staff:
                    ...     show(note)
                    ...
                <abjad_output_block image_layout_specifier="True" image_render_specifier="ImageRenderSpecifier(stylesheet='default.ly')" renderer="lilypond" xml:space="preserve">
                    \version "2.19.0"
                    \language "english"

                    #(ly:set-option 'relative-includes #t)

                    \include "default.ly"

                    \header {
                        tagline = ##f
                    }

                    \score {
                        {
                            c'1
                        }
                    }
                <abjad_output_block image_layout_specifier="True" image_render_specifier="ImageRenderSpecifier(stylesheet='default.ly')" renderer="lilypond" xml:space="preserve">
                    \version "2.19.0"
                    \language "english"

                    #(ly:set-option 'relative-includes #t)

                    \include "default.ly"

                    \header {
                        tagline = ##f
                    }

                    \score {
                        {
                            g'1
                        }
                    }
                <literal_block xml:space="preserve">
                    >>> len(staff)
                    2
            """)
        assert actual == target

    @unittest.skipIf(
        sys.version[0] != '3',
        'Only for Python 3',
        )
    def test_on_doctree_read_03(self):
        source = r'''
        ..  abjad::
            :no-stylesheet:

            staff = Staff("c'1 g'1")
            for note in staff:
                show(note)

            len(staff)
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            r"""
            <document source="test">
                <literal_block xml:space="preserve">
                    >>> staff = Staff("c'1 g'1")
                    >>> for note in staff:
                    ...     show(note)
                    ...
                <abjad_output_block image_layout_specifier="True" image_render_specifier="ImageRenderSpecifier(no_stylesheet=True)" renderer="lilypond" xml:space="preserve">
                    \version "2.19.0"
                    \language "english"

                    \header {
                        tagline = ##f
                    }

                    \score {
                        {
                            c'1
                        }
                    }
                <abjad_output_block image_layout_specifier="True" image_render_specifier="ImageRenderSpecifier(no_stylesheet=True)" renderer="lilypond" xml:space="preserve">
                    \version "2.19.0"
                    \language "english"

                    \header {
                        tagline = ##f
                    }

                    \score {
                        {
                            g'1
                        }
                    }
                <literal_block xml:space="preserve">
                    >>> len(staff)
                    2
            """)
        assert actual == target

    @unittest.skipIf(
        sys.version[0] != '3',
        'Only for Python 3',
        )
    def test_on_doctree_read_04(self):
        source = r'''
        ..  container:: example

            **Example 1.** Duple meter:

            ::

                >>> 1 + 1

            ::

                >>> meter = metertools.Meter((2, 4))
                >>> meter
                Meter('(2/4 (1/4 1/4))')
                >>> print(meter.pretty_rtm_format)
                (2/4 (
                    1/4
                    1/4))

            ::

                >>> graph(meter) # doctest: +SKIP

            `2/4` comprises two beats.

        ..  container:: example

            **Example 2.** Triple meter:

            ::

                >>> meter = metertools.Meter((3, 4))
                >>> print(meter.pretty_rtm_format)
                (3/4 (
                    1/4
                    1/4
                    1/4))

            ::

                >>> graph(meter) # doctest: +SKIP

            `3/4` comprises three beats.
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            r"""
            <document source="test">
                <container classes="example">
                    <paragraph>
                        <strong>
                            Example 1.
                         Duple meter:
                    <literal_block xml:space="preserve">
                        >>> 1 + 1
                        2
                    <literal_block xml:space="preserve">
                        >>> meter = metertools.Meter((2, 4))
                        >>> meter
                        Meter('(2/4 (1/4 1/4))')
                    <literal_block xml:space="preserve">
                        >>> print(meter.pretty_rtm_format)
                        (2/4 (
                            1/4
                            1/4))
                    <literal_block xml:space="preserve">
                        >>> graph(meter) # doctest: +SKIP
                    <abjad_output_block image_layout_specifier="True" image_render_specifier="True" layout="dot" renderer="graphviz" xml:space="preserve">
                        digraph G {
                            graph [bgcolor=transparent,
                                fontname=Arial,
                                penwidth=2,
                                truecolor=true];
                            node [fontname=Arial,
                                fontsize=12,
                                penwidth=2];
                            edge [penwidth=2];
                            node_0 [label="2/4",
                                shape=triangle];
                            node_1 [label="1/4",
                                shape=box];
                            node_2 [label="1/4",
                                shape=box];
                            subgraph cluster_cluster_offsets {
                                graph [style=rounded];
                                node_3_0 [color=white,
                                    fillcolor=black,
                                    fontcolor=white,
                                    fontname="Arial bold",
                                    label="{ <f_0_0> 0 | <f_0_1> ++ }",
                                    shape=Mrecord,
                                    style=filled];
                                node_3_1 [color=white,
                                    fillcolor=black,
                                    fontcolor=white,
                                    fontname="Arial bold",
                                    label="{ <f_0_0> 1/4 | <f_0_1> + }",
                                    shape=Mrecord,
                                    style=filled];
                                node_3_2 [label="{ <f_0_0> 1/2 | <f_0_1> ++ }",
                                    shape=Mrecord];
                            }
                            node_0 -> node_1;
                            node_0 -> node_2;
                            node_1 -> node_3_0 [style=dotted];
                            node_1 -> node_3_1 [style=dotted];
                            node_2 -> node_3_1 [style=dotted];
                            node_2 -> node_3_2 [style=dotted];
                        }
                    <paragraph>
                        <title_reference>
                            2/4
                         comprises two beats.
                <container classes="example">
                    <paragraph>
                        <strong>
                            Example 2.
                         Triple meter:
                    <literal_block xml:space="preserve">
                        >>> meter = metertools.Meter((3, 4))
                        >>> print(meter.pretty_rtm_format)
                        (3/4 (
                            1/4
                            1/4
                            1/4))
                    <literal_block xml:space="preserve">
                        >>> graph(meter) # doctest: +SKIP
                    <abjad_output_block image_layout_specifier="True" image_render_specifier="True" layout="dot" renderer="graphviz" xml:space="preserve">
                        digraph G {
                            graph [bgcolor=transparent,
                                fontname=Arial,
                                penwidth=2,
                                truecolor=true];
                            node [fontname=Arial,
                                fontsize=12,
                                penwidth=2];
                            edge [penwidth=2];
                            node_0 [label="3/4",
                                shape=triangle];
                            node_1 [label="1/4",
                                shape=box];
                            node_2 [label="1/4",
                                shape=box];
                            node_3 [label="1/4",
                                shape=box];
                            subgraph cluster_cluster_offsets {
                                graph [style=rounded];
                                node_4_0 [color=white,
                                    fillcolor=black,
                                    fontcolor=white,
                                    fontname="Arial bold",
                                    label="{ <f_0_0> 0 | <f_0_1> ++ }",
                                    shape=Mrecord,
                                    style=filled];
                                node_4_1 [color=white,
                                    fillcolor=black,
                                    fontcolor=white,
                                    fontname="Arial bold",
                                    label="{ <f_0_0> 1/4 | <f_0_1> + }",
                                    shape=Mrecord,
                                    style=filled];
                                node_4_2 [color=white,
                                    fillcolor=black,
                                    fontcolor=white,
                                    fontname="Arial bold",
                                    label="{ <f_0_0> 1/2 | <f_0_1> + }",
                                    shape=Mrecord,
                                    style=filled];
                                node_4_3 [label="{ <f_0_0> 3/4 | <f_0_1> ++ }",
                                    shape=Mrecord];
                            }
                            node_0 -> node_1;
                            node_0 -> node_2;
                            node_0 -> node_3;
                            node_1 -> node_4_0 [style=dotted];
                            node_1 -> node_4_1 [style=dotted];
                            node_2 -> node_4_1 [style=dotted];
                            node_2 -> node_4_2 [style=dotted];
                            node_3 -> node_4_2 [style=dotted];
                            node_3 -> node_4_3 [style=dotted];
                        }
                    <paragraph>
                        <title_reference>
                            3/4
                         comprises three beats.
            """)
        assert actual == target, \
            systemtools.TestManager.diff(actual, target, 'Diff:')

    def test_on_doctree_read_05(self):
        source = r'''
        ..  abjad::
            :text-width: 40

            [x for x in dir() if not x.startswith('_')]
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            r"""
            <document source="test">
                <literal_block xml:space="preserve">
                    >>> [x for x in dir() if not x.startswith('_')]
                    ['Accelerando', 'Annotation',
                    'Arpeggio', 'Arrow', 'Articulation',
                    'BarLine', 'Beam', 'BendAfter',
                    'BowContactPoint', 'BowContactSpanner',
                    'BowMotionTechnique', 'BowPressure',
                    'BreathMark', 'Chord', 'Clef',
                    'ClefSpanner', 'Cluster',
                    'ColorFingering', 'ComplexBeam',
                    'ComplexTrillSpanner', 'Component',
                    'Container', 'Context', 'Crescendo',
                    'CyclicTuple', 'Decrescendo',
                    'DuratedComplexBeam', 'Duration',
                    'Dynamic', 'Expression', 'Fermata',
                    'FixedDurationTuplet', 'Fraction',
                    'GeneralizedBeam', 'Glissando',
                    'GraceContainer', 'Hairpin',
                    'HiddenStaffSpanner',
                    'HorizontalBracketSpanner', 'Infinity',
                    'Inversion', 'KeyCluster',
                    'KeySignature', 'LaissezVibrer', 'Leaf',
                    'LilyPondCommand', 'LilyPondComment',
                    'LilyPondFile', 'LilyPondLiteral',
                    'LineSegment', 'Markup', 'MarkupList',
                    'Measure', 'MeasuredComplexBeam',
                    'Meter', 'MetricModulation',
                    'MultimeasureRest', 'MultipartBeam',
                    'Multiplication', 'Multiplier',
                    'NamedInterval', 'NamedIntervalClass',
                    'NamedPitch', 'NamedPitchClass',
                    'NonreducedFraction', 'NonreducedRatio',
                    'Note', 'NumberedInterval',
                    'NumberedIntervalClass',
                    'NumberedPitch', 'NumberedPitchClass',
                    'OctavationSpanner', 'Offset',
                    'PageBreak', 'Pattern', 'PhrasingSlur',
                    'PianoPedalSpanner',
                    'PitchClassSegment', 'PitchClassSet',
                    'PitchRange', 'PitchSegment',
                    'PitchSet', 'Ratio', 'Registration',
                    'RehearsalMark', 'Repeat', 'Rest',
                    'Retrograde', 'Ritardando', 'Rotation',
                    'Scheme', 'SchemeMoment', 'SchemePair',
                    'SchemeSymbol', 'SchemeVector', 'Score',
                    'Selection', 'Selector', 'Sequence',
                    'SetClass', 'Skip', 'Slur', 'Spanner',
                    'Staff', 'StaffChange', 'StaffGroup',
                    'StaffLinesSpanner', 'StemTremolo',
                    'StemTremoloSpanner',
                    'StringContactPoint', 'StringNumber',
                    'SystemBreak', 'Tempo', 'TempoSpanner',
                    'TextSpanner', 'Tie', 'TimeSignature',
                    'Timespan', 'Transposition', 'Tremolo',
                    'TrillSpanner', 'Tuning', 'Tuplet',
                    'TwelveToneRow', 'TypedOrderedDict',
                    'Voice', 'abctools',
                    'abjad_configuration', 'agenttools',
                    'attach', 'commandlinetools',
                    'datastructuretools', 'demos', 'detach',
                    'documentationtools', 'durationtools',
                    'exceptiontools', 'expressiontools',
                    'ext', 'f', 'graph', 'graphtools',
                    'indicatortools', 'inspect_',
                    'instrumenttools', 'ipythontools',
                    'iterate', 'label', 'lilypondfiletools',
                    'lilypondnametools',
                    'lilypondparsertools', 'ly',
                    'markuptools', 'mathtools',
                    'metertools', 'mutate', 'new',
                    'override', 'parse', 'patterntools',
                    'persist', 'pitchtools', 'play',
                    'print_function', 'quantizationtools',
                    'rhythmmakertools', 'rhythmtreetools',
                    'schemetools', 'scoretools', 'select',
                    'select_all', 'select_every',
                    'select_first', 'select_last',
                    'selectiontools', 'selectortools',
                    'sequence', 'sequencetools', 'set_',
                    'show', 'silence', 'silence_all',
                    'silence_every', 'silence_except',
                    'silence_first', 'silence_last',
                    'spannertools', 'stringtools',
                    'sustain', 'sustain_all',
                    'sustain_every', 'sustain_first',
                    'sustain_last', 'systemtools',
                    'templatetools', 'timespantools',
                    'tonalanalysistools', 'topleveltools',
                    'tweak']
            """)
        assert actual == target, \
            systemtools.TestManager.diff(actual, target, 'Diff:')

    def test_on_doctree_read_06(self):
        source = u'''
        This example demonstrates the power of exploiting redundancy to model
        musical structure. The piece that concerns us here is Ligeti's *Désordre*:
        the first piano study from Book I. Specifically, we will focus on modeling
        the first section of the piece.

        ..  abjad::

            print('Désordre')
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            u"""
            <document source="test">
                <paragraph>
                    This example demonstrates the power of exploiting redundancy to model
                    musical structure. The piece that concerns us here is Ligeti's
                    <emphasis>
                        Désordre
                    :
                    the first piano study from Book I. Specifically, we will focus on modeling
                    the first section of the piece.
                <literal_block xml:space="preserve">
                    >>> print('Désordre')
                    Désordre
            """)
        assert actual == target, \
            systemtools.TestManager.diff(actual, target, 'Diff:')

    def test_on_doctree_read_07(self):
        source = '''
        ..  abjad::
            :hide:
            :no-trim:
            :pages: 1-4
            :with-columns: 2

            show(Staff("c'4 d'4 e'4 f'4"))
        '''
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            r"""
            <document source="test">
                <block_quote>
                    <abjad_output_block image_layout_specifier="ImageLayoutSpecifier(pages=(1, 2, 3, 4), with_columns=2)" image_render_specifier="ImageRenderSpecifier(no_trim=True, stylesheet='default.ly')" renderer="lilypond" xml:space="preserve">
                        \version "2.19.0"
                        \language "english"

                        #(ly:set-option 'relative-includes #t)

                        \include "default.ly"

                        \header {
                            tagline = ##f
                        }

                        \score {
                            \new Staff {
                                c'4
                                d'4
                                e'4
                                f'4
                            }
                        }
            """)
        assert actual == target, \
            systemtools.TestManager.diff(actual, target, 'Diff:')

    def test_on_doctree_read_08(self):
        source = '''
        ..  abjad::

            staff = Staff("c'4 d'4 e'4 f'4")
            spanner = Slur()
            attach(spanner, staff[:])

        ..  abjad::

            for leaf in staff:
                is_first = spanner._is_my_first_leaf(leaf)
                is_last = spanner._is_my_last_leaf(leaf)
                print(repr(leaf), is_first, is_last)
        '''
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            r"""
            <document source="test">
                <block_quote>
                    <literal_block xml:space="preserve">
                        >>> staff = Staff("c'4 d'4 e'4 f'4")
                        >>> spanner = Slur()
                        >>> attach(spanner, staff[:])
                    <literal_block xml:space="preserve">
                        >>> for leaf in staff:
                        ...     is_first = spanner._is_my_first_leaf(leaf)
                        ...     is_last = spanner._is_my_last_leaf(leaf)
                        ...     print(repr(leaf), is_first, is_last)
                        ...
                        Note("c'4") True False
                        Note("d'4") False False
                        Note("e'4") False False
                        Note("f'4") False True
            """)
        assert actual == target, \
            systemtools.TestManager.diff(actual, target, 'Diff:')

    def test_on_doctree_read_09(self):
        source = '''
        ..  abjad::
            :strip-prompt:

            class Foo(object):

                def bar(self):
                    print('OK')

                def baz(self):
                    print('NO')

                def quux(self):
                    return 23

        ..  abjad::

            foo = Foo()
            foo.quux()
        '''
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            r"""
            <document source="test">
                <block_quote>
                    <literal_block xml:space="preserve">
                        class Foo(object):

                            def bar(self):
                                print('OK')

                            def baz(self):
                                print('NO')

                            def quux(self):
                                return 23
                    <literal_block xml:space="preserve">
                        >>> foo = Foo()
                        >>> foo.quux()
                        23
            """)
        assert actual == target, \
            systemtools.TestManager.diff(actual, target, 'Diff:')

    def test_on_doctree_read_10(self):
        source = '''
        ..  abjad::
            :hide:
            :reveal-label: foo
            :strip-prompt:

            def foo(x):
                return x + 1

        ..  abjad::

            3 * 5

        ..  reveal:: foo

        ..  abjad::

            foo(23)
        '''
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        actual = stringtools.normalize(document.pformat())
        target = stringtools.normalize(
            r"""
            <document source="test">
                <block_quote>
                    <literal_block xml:space="preserve">
                        >>> 3 * 5
                        15
                    <literal_block xml:space="preserve">
                        def foo(x):
                            return x + 1
                    <literal_block xml:space="preserve">
                        >>> foo(23)
                        24
            """)
        assert actual == target, \
            systemtools.TestManager.diff(actual, target, 'Diff:')



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Chord___eq____01():
    '''True only when chords have the same object ID.
    '''

    chord_1 = Chord([0, 4, 7], (1, 4))
    chord_2 = Chord([0, 4, 7], (1, 4))
    chord_3 = Chord([0, 4, 6], (1, 4))

    assert     chord_1 == chord_1
    assert not chord_1 == chord_2
    assert not chord_1 == chord_3
    assert not chord_2 == chord_1
    assert     chord_2 == chord_2
    assert not chord_2 == chord_3
    assert not chord_3 == chord_1
    assert not chord_3 == chord_2
    assert     chord_3 == chord_3



# -*- coding: utf-8 -*-


def inspect_(client):
    r'''Makes inspection agent.

    ..  container:: example

        Example staff:

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                e'4
                d'4
                f'4
            }

    ..  container:: example

        Gets duration of first note in staff:

        ::

            >>> inspect_(staff[0]).get_duration()
            Duration(1, 4)

    ..  container:: example

        Gets lineage of first note in staff:

        ::

            >>> inspect_(staff[0]).get_lineage()
            Lineage([Staff("c'4 e'4 d'4 f'4"), Note("c'4")])

    ..  container:: example

        Returns inspection agent:

        ::

            >>> inspect_(staff)
            InspectionAgent(client=Staff("c'4 e'4 d'4 f'4"))

    '''
    from abjad.tools import agenttools
    return agenttools.InspectionAgent(client=client)



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject


class WrapSelectionCallback(AbjadValueObject):
    r'''Wrap selection callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_apply_to_each',
        )

    ### INITIALIZER ###

    def __init__(self, apply_to_each=None):
        self._apply_to_each = apply_to_each

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Wraps `argument` in selection.

        Somewhat like the opposite of flattening.

        Ignores `rotation`.

        Returns tuple.
        '''
        import abjad
        if isinstance(argument, abjad.Component):
            return (argument,)
        elif isinstance(argument, abjad.Selection):
            if self.apply_to_each:
                selections = [abjad.select([_]) for _ in argument]
                return tuple(selections)
            else:
                selection = abjad.select([argument])
                return (selection,)
        elif isinstance(argument, tuple):
            if self.apply_to_each:
                selections = [abjad.select([_]) for _ in argument]
                return tuple(selections)
            else:
                selection = abjad.select(argument[:])
                return (selection,)
        else:
            message = 'neither component, selection nor tuple: {!r}.'
            message = message.format(argument)
            raise TypeError(message)

    ### PUBLIC PROPERTIES ###

    @property
    def apply_to_each(self):
        r'''Is true when selector maps.

        Returns true, false or none.
        '''
        return self._apply_to_each



# -*- coding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.pitchtools.Vector import Vector


class IntervalVector(Vector):
    r'''Interval vector.

    ..  container:: example

        Initializes from pitch segment:

        ::

            >>> pitch_segment = pitchtools.PitchSegment(
            ...     items=[0, 11, 7, 4, 2, 9, 3, 8, 10, 1, 5, 6],
            ...     )
            >>> numbered_interval_vector = pitchtools.IntervalVector(
            ...     items=pitch_segment,
            ...     item_class=pitchtools.NumberedInterval,
            ...     )
            >>> for interval, count in sorted(numbered_interval_vector.items(),
            ...     key=lambda x: (x[0].direction_number, x[0].number)):
            ...     print(interval, count)
            ...
            -11 1
            -10 1
            -9 1
            -8 2
            -7 3
            -6 3
            -5 4
            -4 4
            -3 4
            -2 5
            -1 6
            +1 5
            +2 5
            +3 5
            +4 4
            +5 3
            +6 3
            +7 2
            +8 2
            +9 2
            +10 1

    '''

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        from abjad.tools import pitchtools
        if isinstance(items, (
            pitchtools.PitchSegment,
            pitchtools.PitchSet,
            pitchtools.PitchClassSegment,
            pitchtools.PitchClassSet,
            )):
            intervals = []
            enumeration = sequencetools.Enumeration(items)
            pairs = enumeration.yield_pairs()
            for first, second in pairs:
                intervals.append(second - first)
            items = intervals
        Vector.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    def __repr__(self):
        r'''Gets interpret representation of interval vector.

        ..  container:: example

            Gets interpreter representation of interval vector:

            ::

                >>> segment = pitchtools.PitchSegment(
                ...     items=[0, 11, 7, 4, 2, 9, 3, 8, 10, 1, 5, 6],
                ...     )
                >>> vector = pitchtools.IntervalVector(
                ...     items=pitch_segment,
                ...     item_class=pitchtools.NumberedInterval,
                ...     )

            ::

                >>> vector
                IntervalVector({-11: 1, -10: 1, -9: 1, -8: 2, -7: 3, -6: 3, -5: 4, -4: 4, -3: 4, -2: 5, -1: 6, 1: 5, 2: 5, 3: 5, 4: 4, 5: 3, 6: 3, 7: 2, 8: 2, 9: 2, 10: 1}, item_class=NumberedInterval)

        ..  container:: example

            Initializes from interpreter representation of interval vector:

                >>> pitchtools.IntervalVector(vector)
                IntervalVector({-11: 1, -10: 1, -9: 1, -8: 2, -7: 3, -6: 3, -5: 4, -4: 4, -3: 4, -2: 5, -1: 6, 1: 5, 2: 5, 3: 5, 4: 4, 5: 3, 6: 3, 7: 2, 8: 2, 9: 2, 10: 1}, item_class=NumberedInterval)

        Returns string.
        '''
        superclass = super(IntervalVector, self)
        return superclass.__repr__()

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedInterval

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedInterval

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.Interval

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        r'''Makes interval vector from `selection`.

        Returns interval vector.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return class_(
            pitch_segment,
            item_class=item_class,
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___01():
    r'''Define LilyPond autoBeaming context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff).auto_beaming = True

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            autoBeaming = ##t
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___02():
    r'''Remove LilyPond autoBeaming context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff).auto_beaming = True
    del(set_(staff).auto_beaming)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___03():
    r'''Define LilyPond currentBarNumber context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff[0]).score.current_bar_number = 12

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Score.currentBarNumber = #12
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___04():
    r'''Define LilyPond currentBarNumber context set_.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    set_(staff[0]).score.current_bar_number = 12

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \set Score.currentBarNumber = #12
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___05():
    r'''Define LilyPond fontSize context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff).font_size = -3

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            fontSize = #-3
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___06():
    r'''Define LilyPond instrumentName context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff).instrument_name = 'Violini I'

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            instrumentName = #"Violini I"
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___07():
    r'''Define LilyPond instrumentName context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff).instrument_name = markuptools.Markup(r'\circle { V }')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            instrumentName = \markup {
                \circle
                    {
                        V
                    }
                }
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___08():
    r'''Define LilyPond proportionalNotationDuration context set_.
    '''

    score = Score([Staff("c'8 d'8 e'8 f'8")])
    moment = schemetools.SchemeMoment(Fraction(1, 56))
    set_(score).proportional_notation_duration = moment

    assert format(score) == stringtools.normalize(
        r'''
        \new Score \with {
            proportionalNotationDuration = #(ly:make-moment 1 56)
        } <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        >>
        '''
        )

    assert inspect_(score).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___09():
    r'''Define LilyPond shortInstrumentName context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff).short_instrument_name = 'Vni. I'

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            shortInstrumentName = #"Vni. I"
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___10():
    r'''Define LilyPond shortInstrumentName context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff).short_instrument_name = markuptools.Markup(
        r'\circle { V }')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            shortInstrumentName = \markup {
                \circle
                    {
                        V
                    }
                }
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___11():
    r'''Define LilyPond suggestAccidentals context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff).suggest_accidentals = True

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            suggestAccidentals = ##t
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___12():
    r'''Define LilyPond suggestAccidentals context set_.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    set_(staff[1]).suggest_accidentals = True

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            \set suggestAccidentals = ##t
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_lilypondproxytools_LilyPondSettingNameManager___setattr___13():
    r'''Define LilyPond tupletFullLength context set_.
    '''

    staff = Staff([])
    set_(staff).tuplet_full_length = True

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            tupletFullLength = ##t
        } {
        }
        '''
        )

    assert not len(staff)

    set_(staff).tuplet_full_length = False

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            tupletFullLength = ##f
        } {
        }
        '''
        )

    assert not len(staff)

    del(set_(staff).tuplet_full_length)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
        }
        '''
        )

    assert not len(staff)


# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_RegistrationComponent___init___01():
    r'''Initialize from range and start pitch.
    '''

    component = pitchtools.RegistrationComponent('[A0, C8]', 15)
    assert isinstance(component, pitchtools.RegistrationComponent)



# -*- coding: utf-8 -*-
from abjad import *


def test_durationtools_Duration_lilypond_duration_string_01():

    assert Duration((1, 16)).lilypond_duration_string == '16'
    assert Duration((2, 16)).lilypond_duration_string == '8'
    assert Duration((3, 16)).lilypond_duration_string == '8.'
    assert Duration((4, 16)).lilypond_duration_string == '4'



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Octave___str___01():


    assert str(pitchtools.Octave(-1)) == ',,,,'
    assert str(pitchtools.Octave(0)) == ',,,'
    assert str(pitchtools.Octave(1)) == ',,'
    assert str(pitchtools.Octave(2)) == ','
    assert str(pitchtools.Octave(3)) == ''
    assert str(pitchtools.Octave(4)) == "'"
    assert str(pitchtools.Octave(5)) == "''"
    assert str(pitchtools.Octave(6)) == "'''"
    assert str(pitchtools.Octave(7)) == "''''"



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment_has_duplicates_01():
    r'''Works with chords.
    '''

    chord = Chord([1, 13, 14], (1, 4))
    assert pitchtools.PitchClassSegment.from_selection(
        chord,
        item_class=pitchtools.NumberedPitchClass,
        ).has_duplicates()

    chord = Chord([1, 14, 15], (1, 4))
    assert not pitchtools.PitchClassSegment.from_selection(
        chord,
        item_class=pitchtools.NumberedPitchClass,
        ).has_duplicates()


def test_pitchtools_PitchClassSegment_has_duplicates_02():
    r'''Works with notes, rests and skips.
    '''

    assert not pitchtools.PitchClassSegment.from_selection(
        Note(13, (1, 4)),
        item_class=pitchtools.NumberedPitchClass,
        ).has_duplicates()
    assert not pitchtools.PitchClassSegment.from_selection(
        Rest((1, 4)),
        item_class=pitchtools.NumberedPitchClass,
        ).has_duplicates()
    assert not pitchtools.PitchClassSegment.from_selection(
        scoretools.Skip((1, 4)),
        item_class=pitchtools.NumberedPitchClass,
        ).has_duplicates()



# -*- coding: utf-8 -*-


def to_snake_case(string):
    r'''Changes `string` to snake case.

    ..  container:: example

        Changes words to snake case:

        ::

            >>> stringtools.to_snake_case('scale degrees 4 and 5')
            'scale_degrees_4_and_5'

    ..  container:: example

        Changes snake case to snake case:

        ::

            >>> stringtools.to_snake_case('scale_degrees_4_and_5')
            'scale_degrees_4_and_5'

    ..  container:: example

        Changes dash case to snake case:

        ::

            >>> stringtools.to_snake_case('scale-degrees-4-and-5')
            'scale_degrees_4_and_5'

    ..  container:: example

        Changes snake case to snake case:

        ::

            >>> stringtools.to_snake_case('ScaleDegrees4And5')
            'scale_degrees_4_and_5'

    Returns string.
    '''
    from abjad.tools import stringtools
    words = stringtools.delimit_words(string)
    words = [_.lower() for _ in words]
    result = '_'.join(words)
    return result



# -*- coding: utf-8 -*-
import re
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadObject


class ChordSuspension(AbjadObject):
    '''A chord of 9-8, 7-6, 4-3, 2-1 and other types of
    suspension typical of, for example, the Bach chorales.

    ::

        >>> suspension = tonalanalysistools.ChordSuspension(4, 3)
        >>> suspension
        ChordSuspension(ScaleDegree(4), ScaleDegree(3))

    Value object that can not be changed after instantiation.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_start',
        '_stop',
        )

    _symbolic_string_regex = re.compile(r'([#|b]?\d+)-([#|b]?\d+)')

    ### INITIALIZER ###

    def __init__(self, *arguments):
        if len(arguments) == 0:
            start, stop = self._initialize_empty()
        elif len(arguments) == 1 and arguments[0] in (None, ''):
            start, stop = self._initialize_empty()
        elif len(arguments) == 1 and isinstance(arguments[0], type(self)):
            start, stop = self._initialize_by_reference(*arguments)
        elif len(arguments) == 1 and isinstance(arguments[0], str):
            start, stop = self._initialize_by_symbolic_string(*arguments)
        elif len(arguments) == 1 and isinstance(arguments[0], tuple):
            start, stop = self._initialize_by_pair(*arguments)
        elif len(arguments) == 2:
            start, stop = self._initialize_by_start_and_stop(*arguments)
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, arguments)
            raise ValueError(message)
        self._start = start
        self._stop = stop

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a chord suspension when start and stop equal
        to those of this chord suspension. Otherwise false.

        Returns true or false.
        '''
        if isinstance(arg, type(self)):
            if self.start == arg.start:
                if self.stop == arg.stop:
                    return True
        return False

    def __hash__(self):
        r'''Hashes chord suspension.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(ChordSuspension, self).__hash__()

    def __ne__(self, arg):
        r'''Is true when `arg` does not equal chord suspension. Otherwise
        false.

        Returns true or false.
        '''
        return not self == arg

    def __str__(self):
        r'''String representation of chord suspension.

        Returns string.
        '''
        if self.start is not None and self.stop is not None:
            return '{!s}-{!s}'.format(self.start, self.stop)
        else:
            return ''

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = []
        if self.start is not None and self.stop is not None:
            values.append(self.start)
            values.append(self.stop)
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=values,
            )

    def _initialize_by_pair(self, pair):
        start, stop = pair
        return self._initialize_by_start_and_stop(start, stop)

    def _initialize_by_reference(self, chord_suspension):
        start, stop = chord_suspension.start, chord_suspension.stop
        return self._initialize_by_start_and_stop(start, stop)

    def _initialize_by_start_and_stop(self, start, stop):
        from abjad.tools import tonalanalysistools
        start = tonalanalysistools.ScaleDegree(start)
        stop = tonalanalysistools.ScaleDegree(stop)
        return start, stop

    def _initialize_by_symbolic_string(self, symbolic_string):
        from abjad.tools import tonalanalysistools
        groups = self._symbolic_string_regex.match(symbolic_string).groups()
        start, stop = groups
        start = tonalanalysistools.ScaleDegree(start)
        stop = tonalanalysistools.ScaleDegree(stop)
        return start, stop

    def _initialize_empty(self):
        return None, None

    ### PUBLIC PROPERTIES ###

    @property
    def chord_name(self):
        r'''Chord name of suspension.

        ::

            >>> suspension.chord_name
            'sus4'

        Returns string.
        '''
        if self.is_empty:
            return ''
        return 'sus{!s}'.format(self.start)

    @property
    def figured_bass_pair(self):
        r'''Figured bass pair of suspension.

        ::

            >>> suspension.figured_bass_pair
            (4, 3)

        Returns integer pair.
        '''
        return self.start.number, self.stop.number

    @property
    def figured_bass_string(self):
        r'''Figured bass string.

        ::

            >>> suspension.figured_bass_string
            '4-3'

        Returns string.
        '''
        if self.is_empty:
            return ''
        return '{!s}-{!s}'.format(self.start, self.stop)

    @property
    def is_empty(self):
        r'''Is true when start and stop are none. Otherwise false.

        ::

            >>> suspension.is_empty
            False

        '''
        return self.start is None and self.stop is None

    @property
    def start(self):
        r'''Start of suspension.

        ::

            >>> suspension.start
            ScaleDegree(4)

        Returns scale degree.
        '''
        return self._start

    @property
    def stop(self):
        r'''Stop of suspension.

        ::

            >>> suspension.stop
            ScaleDegree(3)

        Returns scale degree.
        '''
        return self._stop


    @property
    def title_string(self):
        r'''Title string of suspension.

        ::

            >>> suspension.title_string
            'FourThreeSuspension'

        Returns string.
        '''
        if self.is_empty:
            return ''
        start = self.start.title_string
        stop = self.stop.title_string
        return '{!s}{!s}Suspension'.format(start, stop)



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Hairpin_direction_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    hairpin = Hairpin(descriptor='p < f', direction=Down)
    attach(hairpin, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 _ \< _ \p
            d'8
            e'8
            f'8 _ \f
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_rests_01():
    r'''Make rest.
    '''

    rest = scoretools.make_rests((1, 4))
    assert isinstance(rest, selectiontools.Selection)
    assert len(rest) == 1
    assert isinstance(rest[0], Rest)
    assert rest[0].written_duration == Duration(1, 4)
    assert len(inspect_(rest[0]).get_logical_tie()) == 1


def test_scoretools_make_rests_02():
    r'''Do not tie rests.
    '''

    rest = scoretools.make_rests((5, 8))
    assert len(rest) == 2
    assert isinstance(rest[0], Rest)
    assert isinstance(rest[1], Rest)
    assert rest[0].written_duration == Duration(4, 8)
    assert rest[1].written_duration == Duration(1, 8)
    assert all(len(inspect_(x).get_logical_tie()) == 1 for x in rest)


def test_scoretools_make_rests_03():
    r'''Do not tie rests.
    '''

    true = scoretools.make_rests((5, 8), tie_parts=True)
    assert all(len(inspect_(x).get_logical_tie()) == 1 for x in true)


def test_scoretools_make_rests_04():
    r'''Make rests.
    '''

    t = scoretools.make_rests([(1, 4), Duration(1, 8)])
    assert t[0].written_duration == Duration(1, 4)
    assert t[1].written_duration == Duration(1, 8)
    assert all(len(inspect_(x).get_logical_tie()) == 1 for x in t)



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_QEventSequence_from_tempo_scaled_durations_01():
    r'''Test basic functionality.
    '''

    durations = [durationtools.Duration(x) for x in
        [(1, 4), (1, 3), (1, 7), (2, 5), (3, 4)]]
    tempo = Tempo((1, 4), 55)
    q_events = quantizationtools.QEventSequence.from_tempo_scaled_durations(
        durations, tempo)

    assert q_events == quantizationtools.QEventSequence((
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(12000, 11),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(28000, 11),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(244000, 77),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(34400, 7),
            (NamedPitch("c'"),)
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(630400, 77)
            )
    ))


def test_quantizationtools_QEventSequence_from_tempo_scaled_durations_02():
    r'''Silences are fused.
    '''

    durations = [durationtools.Duration(x) for x in
        [(1, 4), (-1, 4), (1, 4), (1, 4), (-1, 4), (-1, 4), (1, 4)]]
    tempo = Tempo((1, 4), 77)
    q_events = quantizationtools.QEventSequence.from_tempo_scaled_durations(
        durations, tempo)

    assert q_events == quantizationtools.QEventSequence((
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(60000, 77)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(120000, 77),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(180000, 77),
            (NamedPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(240000, 77)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(360000, 77),
            (NamedPitch("c'"),)
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(60000, 11)
            )
    ))



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Container__split_by_duration_01():

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = leaves[0]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=False,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'32 [ (
                c'16.
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_02():

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=False,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 [ (
            }
            {
                \time 7/32
                c'16.
                d'8 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_03():
    '''Split staff. Resulting halves are not well-formed.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff._split_by_duration(
        Duration(1, 32),
        fracture_spanners=False,
        tie_split_notes=False,
        )

    assert format(halves[0][0]) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 [ (
            }
        }
        '''
        ), format(halves[0][0])

    assert format(halves[1][0]) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 7/32
                c'16.
                d'8 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(halves[1][0])

    assert not inspect_(halves[0][0]).is_well_formed()
    assert not inspect_(halves[1][0]).is_well_formed()


def test_scoretools_Container__split_by_duration_04():
    r'''Split one leaf in score.
    Do not fracture spanners. But do tie after split.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = leaves[0]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=False,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'32 ~ [ (
                c'16.
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_05():
    r'''Split one measure in score.
    Do not fracture spanners. But do add tie after split.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=False,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 ~ [ (
            }
            {
                \time 7/32
                c'16.
                d'8 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_06():
    r'''Split in-score measure with power-of-two time signature denominator
    at split offset without power-of-two denominator.
    Do not fracture spanners and do not tie leaves after split.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 5),
        fracture_spanners=False,
        )

    # TODO: The tie at the split locus here is a (small) bug.
    #       Eventually should fix.
    #       The tie after the d'16. is the incorrect one.
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/20
                \scaleDurations #'(4 . 5) {
                    c'8 ~ [ (
                    c'32
                    d'16. ~
                }
            }
            {
                \time 1/20
                \scaleDurations #'(4 . 5) {
                    d'16 ]
                }
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_07():
    r'''Split in-score measure with power-of-two time signature denominator
    at split offset without power-of-two denominator.
    Do fracture spanners and do tie leaves after split.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 5),
        fracture_spanners=False,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/20
                \scaleDurations #'(4 . 5) {
                    c'8 ~ [ (
                    c'32
                    d'16. ~
                }
            }
            {
                \time 1/20
                \scaleDurations #'(4 . 5) {
                    d'16 ]
                }
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_08():
    r'''Split leaf in score and fracture spanners.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = leaves[0]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'32 [ ]
                c'16. [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_09():
    r'''Split measure in score and fracture spanners.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 [ ]
            }
            {
                \time 7/32
                c'16. [ (
                d'8 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_10():
    r'''Split staff outside of score and fracture spanners.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(halves[0][0]) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 [ ]
            }
        }
        '''
        ), format(halves[0][0])

    assert format(halves[1][0]) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 7/32
                c'16. [ (
                d'8 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(halves[1][0])


def test_scoretools_Container__split_by_duration_11():
    r'''Split leaf in score at nonzero index.
    Fracture spanners.
    Test comes from a bug fix.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = leaves[1]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'32 ] )
                d'16. [ ] (
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_12():
    r'''Split container over leaf at nonzero index.
    Fracture spanners.
    Test results from bug fix.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(7, 32),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 7/32
                c'8 [ (
                d'16. ] )
            }
            {
                \time 1/32
                d'32 [ ] (
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_13():
    r'''Split container between leaves and fracture spanners.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 8),
        fracture_spanners=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/8
                c'8 [ ]
            }
            {
                d'8 [ ] (
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_14():
    r'''Split leaf outside of score and fracture spanners.
    '''

    note = Note(0, (1, 8))
    beam = Beam()
    attach(beam, note)

    assert format(note) == "c'8 [ ]"

    halves = note._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        )

    assert format(halves[0][0]) == "c'32 ~ [ ]"
    assert inspect_(halves[0][0]).is_well_formed()

    assert format(halves[1][0]) == "c'16. [ ]"
    assert inspect_(halves[1][0]).is_well_formed()


def test_scoretools_Container__split_by_duration_15():
    r'''Split leaf in score and fracture spanners.
    Tie leaves after split.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = leaves[0]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'32 ~ [ ]
                c'16. [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_16():
    r'''Split measure in score and fracture spanners.
    Tie leaves after split.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 ~ [ ]
            }
            {
                \time 7/32
                c'16. [ (
                d'8 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_17():
    r'''Split in-score measure with power-of-two time signature denominator
    at split offset without power-of-two denominator.
    Do fracture spanners but do not tie leaves after split.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 5),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/20
                \scaleDurations #'(4 . 5) {
                    c'8 ~ [ (
                    c'32
                    d'16. ] )
                }
            }
            {
                \time 1/20
                \scaleDurations #'(4 . 5) {
                    d'16 [ ] (
                }
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_18():
    r'''Split in-score measure with power-of-two time signature denominator at
    split offset without power-of-two denominator.
    Do fracture spanners and do tie leaves after split.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 5),
        fracture_spanners=True,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/20
                \scaleDurations #'(4 . 5) {
                    c'8 ~ [ (
                    c'32
                    d'16. ~ ] )
                }
            }
            {
                \time 1/20
                \scaleDurations #'(4 . 5) {
                    d'16 [ ] (
                }
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_19():
    r'''Split measure with power-of-two time signature denominator at
    split offset without power-of-two denominator.
    Do fracture spanners but do not tie across split locus.
    This test results from a fix.
    What's being tested here is contents rederivation.
    '''

    staff = Staff()
    staff.append(Measure((3, 8), "c'8 d'8 e'8"))
    staff.append(Measure((3, 8), "c'8 d'8 e'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:3])
    beam = Beam()
    attach(beam, leaves[-3:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/8
                c'8 [ (
                d'8
                e'8 ]
            }
            {
                c'8 [
                d'8
                e'8 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(7, 20),
        fracture_spanners=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 14/40
                \scaleDurations #'(4 . 5) {
                    c'8 ~ [ (
                    c'32
                    d'8 ~
                    d'32
                    e'8 ] )
                }
            }
            {
                \time 1/40
                \scaleDurations #'(4 . 5) {
                    e'32 [ ] (
                }
            }
            {
                \time 3/8
                c'8 [
                d'8
                e'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_20():
    r'''Split leaf with LilyPond multiplier.
    Split at split offset with power-of-two denominator.
    Halves carry original written duration.
    Halves carry adjusted LilyPond multipliers.
    '''

    note = Note(0, (1, 8))
    attach(Multiplier(1, 2), note)

    assert format(note) == "c'8 * 1/2"

    halves = note._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(halves[0][0]) == "c'8 * 1/4"
    assert format(halves[1][0]) == "c'8 * 1/4"

    assert inspect_(halves[0][0]).is_well_formed()
    assert inspect_(halves[1][0]).is_well_formed()


def test_scoretools_Container__split_by_duration_21():
    r'''Split leaf with LilyPond multiplier.
    Split at offset without power-of-two denominator.
    Halves carry original written duration.
    Halves carry adjusted LilyPond multipliers.
    '''

    note = Note(0, (1, 8))
    attach(Multiplier(1, 2), note)

    assert format(note) == "c'8 * 1/2"

    halves = note._split_by_duration(
        Duration(1, 48),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(halves[0][0]) == "c'8 * 1/6"
    assert format(halves[1][0]) == "c'8 * 1/3"

    assert inspect_(halves[0][0]).is_well_formed()
    assert inspect_(halves[1][0]).is_well_formed()


def test_scoretools_Container__split_by_duration_22():
    r'''Split measure with power-of-two time signature denominator
    with multiplied leaes. Split at between-leaf offset with
    power-of-two denominator. Leaves remain unaltered.
    '''

    staff = Staff()
    staff.append(Measure((2, 16), "c'8 d'8"))
    staff.append(Measure((2, 16), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    for leaf in leaves:
        attach(Multiplier(1, 2), leaf)
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/16
                c'8 * 1/2 [ (
                d'8 * 1/2 ]
            }
            {
                e'8 * 1/2 [
                f'8 * 1/2 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(1, 16),
        fracture_spanners=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/16
                c'8 * 1/2 [ ]
            }
            {
                d'8 * 1/2 [ ] (
            }
            {
                \time 2/16
                e'8 * 1/2 [
                f'8 * 1/2 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_23():
    r'''Split measure with power-of-two time signature denominator
    with multiplied leaves. Split at through-leaf offset with
    power-of-two denominator. Leaf written durations stay the same
    but multipliers change.
    '''

    staff = Staff()
    staff.append(Measure((2, 16), "c'8 d'8"))
    staff.append(Measure((2, 16), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    for leaf in leaves:
        attach(Multiplier(1, 2), leaf)
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/16
                c'8 * 1/2 [ (
                d'8 * 1/2 ]
            }
            {
                e'8 * 1/2 [
                f'8 * 1/2 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(3, 32),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/32
                c'8 * 1/2 [ (
                d'8 * 1/4 ] )
            }
            {
                \time 1/32
                d'8 * 1/4 [ ] (
            }
            {
                \time 2/16
                e'8 * 1/2 [
                f'8 * 1/2 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_24():
    r'''Split measure with power-of-two time signature denominator
    with multiplied leaves. Split at through-leaf offset without
    power-of-two denominator. Leaf written durations adjust for change
    from power-of-two denominator to non-power-of-two denominator.
    Leaf multipliers also change.
    '''

    staff = Staff()
    staff.append(Measure((2, 16), "c'8 d'8"))
    staff.append(Measure((2, 16), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    for leaf in leaves:
        attach(Multiplier(1, 2), leaf)
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/16
                c'8 * 1/2 [ (
                d'8 * 1/2 ]
            }
            {
                e'8 * 1/2 [
                f'8 * 1/2 ] )
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(2, 24),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/24
                \scaleDurations #'(2 . 3) {
                    c'8. * 1/2 [ (
                    d'8. * 1/6 ] )
                }
            }
            {
                \time 1/24
                \scaleDurations #'(2 . 3) {
                    d'8. * 1/3 [ ] (
                }
            }
            {
                \time 2/16
                e'8 * 1/2 [
                f'8 * 1/2 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_25():
    r'''Split measure with power-of-two time signature denominator
    with multiplied leaves. Time signature carries numerator that
    necessitates ties. Split at through-leaf offset without
    power-of-two denominator.
    '''

    staff = Staff([Measure((5, 16), "s1 * 5/16")])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/16
                s1 * 5/16
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(16, 80),
        fracture_spanners=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 16/80
                \scaleDurations #'(4 . 5) {
                    s1 * 1/4
                }
            }
            {
                \time 9/80
                \scaleDurations #'(4 . 5) {
                    s1 * 1/16
                    s4 * 5/16
                }
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_26():
    r'''Split measure without power-of-two time signature denominator
    at split offset without power-of-two denominator.
    Measure multiplier and split offset multiplier match.
    Split between leaves but do fracture spanners.
    '''

    measure = Measure((15, 80), "c'32 d' e' f' g' a' b' c''64")
    measure.implicit_scaling = True
    staff = Staff([measure])
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 15/80
                \scaleDurations #'(4 . 5) {
                    c'32 [ (
                    d'32
                    e'32
                    f'32
                    g'32
                    a'32
                    b'32
                    c''64 ] )
                }
            }
        }
        '''
        ), format(staff)

    halves = staff[0]._split_by_duration(
        Duration(14, 80),
        fracture_spanners=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 14/80
                \scaleDurations #'(4 . 5) {
                    c'32 [ (
                    d'32
                    e'32
                    f'32
                    g'32
                    a'32
                    b'32 ] )
                }
            }
            {
                \time 1/80
                \scaleDurations #'(4 . 5) {
                    c''64 [ ]
                }
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container__split_by_duration_27():
    r'''Make sure tie (re)application happens only where sensible.
    '''

    halves = Container("c'4")._split_by_duration(
        Duration(3, 16),
        fracture_spanners=True,
        )

    assert format(halves[0][0]) == stringtools.normalize(
        r'''
        {
            c'8.
        }
        '''
        ), format(halves[0][0])

    assert format(halves[-1][0]) == stringtools.normalize(
        r'''
        {
            c'16
        }
        '''
        ), format(halves[-1][0])

    assert inspect_(halves[0][0]).is_well_formed()
    assert inspect_(halves[-1][0]).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList


class MeterList(TypedList):
    r'''Meter list.

    ..  container:: example

        ::

            >>> meters = metertools.MeterList([
            ...     (3, 4), (5, 16), (7, 8),
            ...     ])

        ::

            >>> f(meters)
            metertools.MeterList(
                [
                    metertools.Meter(
                        '(3/4 (1/4 1/4 1/4))'
                        ),
                    metertools.Meter(
                        '(5/16 ((3/16 (1/16 1/16 1/16)) (2/16 (1/16 1/16))))'
                        ),
                    metertools.Meter(
                        '(7/8 ((3/8 (1/8 1/8 1/8)) (2/8 (1/8 1/8)) (2/8 (1/8 1/8))))'
                        ),
                    ]
                )

        ::

            >>> show(meters, scale=0.5) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __illustrate__(self, denominator=16, range_=None, scale=None):
        r'''Illustrates meters.

        ..  container:: example

            ::

                >>> meters = metertools.MeterList([
                ...     (3, 4), (5, 16), (7, 8),
                ...     ])
                >>> show(meters, scale=0.5) # doctest: +SKIP

            ..  doctest

                >>> lilypond_file = meters.__illustrate__()
                >>> f(lilypond_file)
                % ...
                <BLANKLINE>
                \version "..."
                \language "english"
                <BLANKLINE>
                \header {
                    tagline = ##f
                }
                <BLANKLINE>
                \layout {}
                <BLANKLINE>
                \paper {}
                <BLANKLINE>
                \markup {
                    \column
                        {
                            \combine
                                \combine
                                    \translate
                                        #'(1.0 . 1)
                                        \sans
                                            \fontsize
                                                #-3
                                                \center-align
                                                    \fraction
                                                        3
                                                        4
                                    \translate
                                        #'(49.387... . 1)
                                        \sans
                                            \fontsize
                                                #-3
                                                \center-align
                                                    \fraction
                                                        5
                                                        16
                                \translate
                                    #'(69.548... . 1)
                                    \sans
                                        \fontsize
                                            #-3
                                            \center-align
                                                \fraction
                                                    7
                                                    8
                            \combine
                                \postscript
                                    #"
                                    0.2 setlinewidth
                                    1 0.5 moveto
                                    49.387... 0.5 lineto
                                    stroke
                                    1 1.25 moveto
                                    1 -0.25 lineto
                                    stroke
                                    49.387... 1.25 moveto
                                    49.387... -0.25 lineto
                                    stroke
                                    49.387... 0.5 moveto
                                    69.548... 0.5 lineto
                                    stroke
                                    49.387... 1.25 moveto
                                    49.387... -0.25 lineto
                                    stroke
                                    69.548... 1.25 moveto
                                    69.548... -0.25 lineto
                                    stroke
                                    69.548... 0.5 moveto
                                    126 0.5 lineto
                                    stroke
                                    69.548... 1.25 moveto
                                    69.548... -0.25 lineto
                                    stroke
                                    126 1.25 moveto
                                    126 -0.25 lineto
                                    stroke
                                    "
                                \postscript
                                    #"
                                    1 -2 moveto
                                    0 -6.153... rlineto
                                    stroke
                                    5.032... -2 moveto
                                    0 -1.538... rlineto
                                    stroke
                                    9.064... -2 moveto
                                    0 -3.076... rlineto
                                    stroke
                                    13.096... -2 moveto
                                    0 -1.538... rlineto
                                    stroke
                                    17.129... -2 moveto
                                    0 -4.615... rlineto
                                    stroke
                                    21.161... -2 moveto
                                    0 -1.538... rlineto
                                    stroke
                                    25.193... -2 moveto
                                    0 -3.076... rlineto
                                    stroke
                                    29.225... -2 moveto
                                    0 -1.538... rlineto
                                    stroke
                                    33.258... -2 moveto
                                    0 -4.615... rlineto
                                    stroke
                                    37.290... -2 moveto
                                    0 -1.538... rlineto
                                    stroke
                                    41.322... -2 moveto
                                    0 -3.076... rlineto
                                    stroke
                                    45.354... -2 moveto
                                    0 -1.538... rlineto
                                    stroke
                                    49.387... -2 moveto
                                    0 -6.153... rlineto
                                    stroke
                                    49.387... -2 moveto
                                    0 -10.909... rlineto
                                    stroke
                                    53.419... -2 moveto
                                    0 -3.636... rlineto
                                    stroke
                                    57.451... -2 moveto
                                    0 -3.636... rlineto
                                    stroke
                                    61.483... -2 moveto
                                    0 -7.272... rlineto
                                    stroke
                                    65.516... -2 moveto
                                    0 -3.636... rlineto
                                    stroke
                                    69.548... -2 moveto
                                    0 -10.909... rlineto
                                    stroke
                                    69.548... -2 moveto
                                    0 -5.517... rlineto
                                    stroke
                                    73.580... -2 moveto
                                    0 -1.379... rlineto
                                    stroke
                                    77.612... -2 moveto
                                    0 -2.758... rlineto
                                    stroke
                                    81.645... -2 moveto
                                    0 -1.379... rlineto
                                    stroke
                                    85.677... -2 moveto
                                    0 -2.758... rlineto
                                    stroke
                                    89.709... -2 moveto
                                    0 -1.379... rlineto
                                    stroke
                                    93.741... -2 moveto
                                    0 -4.137... rlineto
                                    stroke
                                    97.774... -2 moveto
                                    0 -1.379... rlineto
                                    stroke
                                    101.806... -2 moveto
                                    0 -2.758... rlineto
                                    stroke
                                    105.838... -2 moveto
                                    0 -1.379... rlineto
                                    stroke
                                    109.870... -2 moveto
                                    0 -4.137... rlineto
                                    stroke
                                    113.903... -2 moveto
                                    0 -1.379... rlineto
                                    stroke
                                    117.935... -2 moveto
                                    0 -2.758... rlineto
                                    stroke
                                    121.967... -2 moveto
                                    0 -1.379... rlineto
                                    stroke
                                    126 -2 moveto
                                    0 -5.517... rlineto
                                    stroke
                                    "
                        }
                    }

        Returns LilyPond file.
        '''
        import abjad
        durations = [_.duration for _ in self]
        total_duration = sum(durations)
        offsets = abjad.mathtools.cumulative_sums(durations, start=0)
        timespans = abjad.timespantools.TimespanList()
        for one, two in abjad.Sequence(offsets).nwise():
            timespan = abjad.Timespan(
                start_offset=one,
                stop_offset=two,
                )
            timespans.append(timespan)
        if range_ is not None:
            minimum, maximum = range_
        else:
            minimum, maximum = 0, total_duration
        minimum = float(abjad.Offset(minimum))
        maximum = float(abjad.Offset(maximum))
        if scale is None:
            scale = 1.
        assert 0 < scale
        postscript_scale = 125. / (maximum - minimum)
        postscript_scale *= float(scale)
        postscript_x_offset = (minimum * postscript_scale) - 1
        timespan_markup = timespans._make_timespan_list_markup(
            timespans,
            postscript_x_offset,
            postscript_scale,
            draw_offsets=False,
            )
        ps = abjad.markuptools.Postscript()
        rational_x_offset = abjad.Offset(0)
        for meter in self:
            kernel_denominator = denominator or meter.denominator
            kernel = abjad.metertools.MetricAccentKernel.from_meter(
                meter, kernel_denominator)
            for offset, weight in sorted(kernel.kernel.items()):
                weight = float(weight) * -40
                ps_x_offset = float(rational_x_offset + offset)
                ps_x_offset *= postscript_scale
                ps_x_offset += 1
                ps = ps.moveto(ps_x_offset, -2)
                ps = ps.rlineto(0, weight)
                ps = ps.stroke()
            rational_x_offset += meter.duration
        ps = abjad.Markup.postscript(ps)
        markup_list = abjad.MarkupList([timespan_markup, ps])
        lines_markup = markup_list.combine()
        fraction_markups = []
        for meter, offset in zip(self, offsets):
            numerator, denominator = meter.numerator, meter.denominator
            fraction = abjad.Markup.fraction(numerator, denominator)
            fraction = fraction.center_align().fontsize(-3).sans()
            x_translation = (float(offset) * postscript_scale)
            x_translation -= postscript_x_offset
            fraction = fraction.translate((x_translation, 1))
            fraction_markups.append(fraction)
        fraction_markup = fraction_markups[0]
        for markup in fraction_markups[1:]:
            markup_list = [fraction_markup, markup]
            markup_list = abjad.MarkupList(markup_list)
            fraction_markup = markup_list.combine()
        markup = abjad.Markup.column([fraction_markup, lines_markup])
        return markup.__illustrate__()

    ### PRIVATE PROPERTIES ###

    @property
    def _item_coercer(self):
        from abjad.tools import metertools
        return metertools.Meter



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NamedPitch___slots___01():

    named_pitch = NamedPitch("cs''")
    assert pytest.raises(AttributeError, "named_pitch.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Component__is_immediate_temporal_successor_of_01():

    voice = Voice("c'8 d'8 e'8 f'8")

    assert voice[0]._is_immediate_temporal_successor_of(voice[1])
    assert voice[1]._is_immediate_temporal_successor_of(voice[2])
    assert voice[2]._is_immediate_temporal_successor_of(voice[3])


def test_scoretools_Component__is_immediate_temporal_successor_of_02():

    staff = Staff("c'8 d'8 e'8 f'8")

    assert staff[0]._is_immediate_temporal_successor_of(staff[1])
    assert staff[1]._is_immediate_temporal_successor_of(staff[2])
    assert staff[2]._is_immediate_temporal_successor_of(staff[3])


def test_scoretools_Component__is_immediate_temporal_successor_of_03():

    container = Container("c'8 d'8 e'8 f'8")

    assert container[0]._is_immediate_temporal_successor_of(container[1])
    assert container[1]._is_immediate_temporal_successor_of(container[2])
    assert container[2]._is_immediate_temporal_successor_of(container[3])


def test_scoretools_Component__is_immediate_temporal_successor_of_04():

    tuplet = Tuplet((2, 3), "c'8 d'8 e'8")

    assert tuplet[0]._is_immediate_temporal_successor_of(tuplet[1])
    assert tuplet[1]._is_immediate_temporal_successor_of(tuplet[2])


def test_scoretools_Component__is_immediate_temporal_successor_of_05():

    voice = Voice("{ c'8 d'8 e'8 f'8 } { g'8 a'8 b'8 c''8 }")

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                d'8
                e'8
                f'8
            }
            {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    assert voice[0]._is_immediate_temporal_successor_of(voice[1])
    assert voice[0]._is_immediate_temporal_successor_of(voice[1][0])
    assert voice[0][-1]._is_immediate_temporal_successor_of(voice[1])
    assert voice[0][-1]._is_immediate_temporal_successor_of(voice[1][0])


def test_scoretools_Component__is_immediate_temporal_successor_of_06():

    voice = Voice(r"\times 2/3 { c'8 d'8 e'8 } \times 2/3 { f'8 e'8 d'8 }")

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            \times 2/3 {
                f'8
                e'8
                d'8
            }
        }
        '''
        )

    assert voice[0]._is_immediate_temporal_successor_of(voice[1])
    assert voice[0]._is_immediate_temporal_successor_of(voice[1][0])
    assert voice[0][-1]._is_immediate_temporal_successor_of(voice[1])
    assert voice[0][-1]._is_immediate_temporal_successor_of(voice[1][0])



def test_scoretools_Component__is_immediate_temporal_successor_of_07():

    staff = Staff([Voice("c'8 d'8 e'8 f'8"), Voice("g'8 a'8 b'8 c''8")])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                c'8
                d'8
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    assert staff[0]._is_immediate_temporal_successor_of(staff[1])
    assert staff[0]._is_immediate_temporal_successor_of(staff[1][0])
    assert staff[0][-1]._is_immediate_temporal_successor_of(staff[1])
    assert staff[0][-1]._is_immediate_temporal_successor_of(staff[1][0])


def test_scoretools_Component__is_immediate_temporal_successor_of_08():

    staff = Staff([Voice("c'8 d'8 e'8 f'8"), Voice("g'8 a'8 b'8 c''8")])
    staff[0].name = 'foo'
    staff[1].name = 'foo'

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "foo" {
                c'8
                d'8
                e'8
                f'8
            }
            \context Voice = "foo" {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    assert staff[0]._is_immediate_temporal_successor_of(staff[1])
    assert staff[0]._is_immediate_temporal_successor_of(staff[1][0])
    assert staff[0][-1]._is_immediate_temporal_successor_of(staff[1])



def test_scoretools_Component__is_immediate_temporal_successor_of_09():

    staff = Staff([Voice("c'8 d'8 e'8 f'8"), Voice("g'8 a'8 b'8 c''8")])
    staff[0].name = 'foo'
    staff[1].name = 'bar'

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "foo" {
                c'8
                d'8
                e'8
                f'8
            }
            \context Voice = "bar" {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    assert staff[0]._is_immediate_temporal_successor_of(staff[1])
    assert staff[0]._is_immediate_temporal_successor_of(staff[1][0])
    assert staff[0][-1]._is_immediate_temporal_successor_of(staff[1])


def test_scoretools_Component__is_immediate_temporal_successor_of_10():

    staff_1 = Staff([Voice("c'8 d'8 e'8 f'8")])
    staff_2 = Staff([Voice("g'8 a'8 b'8 c''8")])
    container = Container([staff_1, staff_2])

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Staff {
                \new Voice {
                    c'8
                    d'8
                    e'8
                    f'8
                }
            }
            \new Staff {
                \new Voice {
                    g'8
                    a'8
                    b'8
                    c''8
                }
            }
        }
        '''
        )

    assert staff_1._is_immediate_temporal_successor_of(staff_2)
    assert staff_1._is_immediate_temporal_successor_of(staff_2[0])
    assert staff_1._is_immediate_temporal_successor_of(staff_2[0][0])

    assert staff_1[0]._is_immediate_temporal_successor_of(staff_2)
    assert staff_1[0]._is_immediate_temporal_successor_of(staff_2[0])
    assert staff_1[0]._is_immediate_temporal_successor_of(staff_2[0][0])

    assert staff_1[0][-1]._is_immediate_temporal_successor_of(staff_2)
    assert staff_1[0][-1]._is_immediate_temporal_successor_of(staff_2[0])
    assert staff_1[0][-1]._is_immediate_temporal_successor_of(staff_2[0][0])


def test_scoretools_Component__is_immediate_temporal_successor_of_11():

    upper_voice_1 = Voice("c''8 d''8 e''8 f''8")
    upper_voice_2 = Voice("g''8 a''8 b''8 c''8")
    lower_voice_1 = Voice("c'8 d'8 e'8 f'8")
    lower_voice_2 = Voice("g'8 a'8 b'8 c''8")
    staff_1 = Staff([upper_voice_1, lower_voice_1])
    staff_2 = Staff([upper_voice_2, lower_voice_2])
    staff_1.is_simultaneous = True
    staff_2.is_simultaneous = True
    container = Container([staff_1, staff_2])

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Staff <<
                \new Voice {
                    c''8
                    d''8
                    e''8
                    f''8
                }
                \new Voice {
                    c'8
                    d'8
                    e'8
                    f'8
                }
            >>
            \new Staff <<
                \new Voice {
                    g''8
                    a''8
                    b''8
                    c''8
                }
                \new Voice {
                    g'8
                    a'8
                    b'8
                    c''8
                }
            >>
        }
        '''
        )

    assert staff_1._is_immediate_temporal_successor_of(staff_2)
    assert staff_1._is_immediate_temporal_successor_of(staff_2[0])
    assert staff_1._is_immediate_temporal_successor_of(staff_2[0][0])
    assert staff_1._is_immediate_temporal_successor_of(staff_2[1])
    assert staff_1._is_immediate_temporal_successor_of(staff_2[1][0])

    assert staff_1[0]._is_immediate_temporal_successor_of(staff_2)
    assert staff_1[0]._is_immediate_temporal_successor_of(staff_2[0])
    assert staff_1[0]._is_immediate_temporal_successor_of(staff_2[0][0])
    assert staff_1[0]._is_immediate_temporal_successor_of(staff_2[1])
    assert staff_1[0]._is_immediate_temporal_successor_of(staff_2[1][0])

    assert staff_1[0][-1]._is_immediate_temporal_successor_of(staff_2)
    assert staff_1[0][-1]._is_immediate_temporal_successor_of(staff_2[0])
    assert staff_1[0][-1]._is_immediate_temporal_successor_of(staff_2[0][0])
    assert staff_1[0][-1]._is_immediate_temporal_successor_of(staff_2[1])
    assert staff_1[0][-1]._is_immediate_temporal_successor_of(staff_2[1][0])

    assert staff_1[1]._is_immediate_temporal_successor_of(staff_2)
    assert staff_1[1]._is_immediate_temporal_successor_of(staff_2[0])
    assert staff_1[1]._is_immediate_temporal_successor_of(staff_2[0][0])
    assert staff_1[1]._is_immediate_temporal_successor_of(staff_2[1])
    assert staff_1[1]._is_immediate_temporal_successor_of(staff_2[1][0])

    assert staff_1[1][-1]._is_immediate_temporal_successor_of(staff_2)
    assert staff_1[1][-1]._is_immediate_temporal_successor_of(staff_2[0])
    assert staff_1[1][-1]._is_immediate_temporal_successor_of(staff_2[0][0])
    assert staff_1[1][-1]._is_immediate_temporal_successor_of(staff_2[1])
    assert staff_1[1][-1]._is_immediate_temporal_successor_of(staff_2[1][0])


def test_scoretools_Component__is_immediate_temporal_successor_of_12():

    voice = Voice("{ { c'8 d'8 e'8 f'8 } } { { g'8 a'8 b'8 c''8 } }")

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                {
                    c'8
                    d'8
                    e'8
                    f'8
                }
            }
            {
                {
                    g'8
                    a'8
                    b'8
                    c''8
                }
            }
        }
        '''
        )

    assert voice[0]._is_immediate_temporal_successor_of(voice[1])
    assert voice[0]._is_immediate_temporal_successor_of(voice[1][0])
    assert voice[0]._is_immediate_temporal_successor_of(voice[1][0][0])

    assert voice[0][0]._is_immediate_temporal_successor_of(voice[1])
    assert voice[0][0]._is_immediate_temporal_successor_of(voice[1][0])
    assert voice[0][0]._is_immediate_temporal_successor_of(voice[1][0][0])

    assert voice[0][0][-1]._is_immediate_temporal_successor_of(voice[1])
    assert voice[0][0][-1]._is_immediate_temporal_successor_of(voice[1][0])
    assert voice[0][0][-1]._is_immediate_temporal_successor_of(voice[1][0][0])



# -*- coding: utf-8 -*-
import abc
import copy
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools import systemtools
from abjad.tools import timespantools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import select
from abjad.tools.topleveltools import set_
from abjad.tools.scoretools.Component import Component


class Leaf(Component):
    r'''Abstract base class from which leaves inherit.

    Leaves include notes, rests, chords and skips.
    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        '_leaf_index',
        '_written_duration',
        )

    _is_counttime_component = True

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, written_duration, name=None):
        Component.__init__(self, name=name)
        self._leaf_index = None
        self.written_duration = durationtools.Duration(written_duration)

    ### SPECIAL METHODS ###

    def __getnewargs__(self):
        '''Gets new arguments.

        Returns tuple.
        '''
        return (self.written_duration,)

    def __str__(self):
        '''String representation of leaf.

        Returns string.
        '''
        return self._compact_representation

    ### PRIVATE METHODS ###

    def _as_graphviz_node(self):
        from abjad.tools import graphtools
        lilypond_format = self._compact_representation
        lilypond_format = lilypond_format.replace('<', '&lt;')
        lilypond_format = lilypond_format.replace('>', '&gt;')
        node = Component._as_graphviz_node(self)
        node[0].extend([
            graphtools.GraphvizTableRow([
                graphtools.GraphvizTableCell(
                    label=type(self).__name__,
                    attributes={'border': 0},
                    ),
                ]),
            graphtools.GraphvizTableHorizontalRule(),
            graphtools.GraphvizTableRow([
                graphtools.GraphvizTableCell(
                    label=lilypond_format,
                    attributes={'border': 0},
                    ),
                ]),
            ])
        return node

    def _copy_override_and_set_from_leaf(self, leaf):
        if getattr(leaf, '_lilypond_grob_name_manager', None) is not None:
            self._lilypond_grob_name_manager = copy.copy(override(leaf))
        if getattr(leaf, '_lilypond_setting_name_manager', None) is not None:
            self._lilypond_setting_name_manager = copy.copy(
                set_(leaf))
        new_indicators = []
        for indicator in leaf._indicator_expressions:
            new_indicator = copy.copy(indicator)
            new_indicators.append(new_indicator)
        for new_indicator in new_indicators:
            attach(new_indicator, self)

    def _copy_with_indicators_but_without_children_or_spanners(self):
        new = Component._copy_with_indicators_but_without_children_or_spanners(self)
        for grace_container in self._get_grace_containers():
            new_grace_container = \
                grace_container._copy_with_children_and_indicators_but_without_spanners()
            attach(new_grace_container, new)
        return new

    def _format_after_grace_body(self):
        result = []
        if self._after_grace is not None:
            after_grace = self._after_grace
            if len(after_grace):
                result.append(format(after_grace))
        return ['after grace body', result]

    def _format_after_grace_opening(self):
        result = []
        if self._after_grace is not None:
            if len(self._after_grace):
                result.append(r'\afterGrace')
        return ['after grace opening', result]

    def _format_after_slot(self, bundle):
        result = []
        result.append(('spanners', bundle.after.spanners))
        result.append(('grob reverts', bundle.grob_reverts))
        result.append(('indicators', bundle.after.indicators))
        result.append(('commands', bundle.after.commands))
        result.append(('comments', bundle.after.comments))
        return result

    def _format_before_slot(self, bundle):
        result = []
        result.append(self._format_grace_body())
        result.append(('comments', bundle.before.comments))
        result.append(('commands', bundle.before.commands))
        result.append(('indicators', bundle.before.indicators))
        result.append(('grob overrides', bundle.grob_overrides))
        result.append(('context settings', bundle.context_settings))
        result.append(('spanners', bundle.before.spanners))
        return result

    def _format_close_brackets_slot(self, bundle):
        return []

    def _format_closing_slot(self, bundle):
        result = []
        result.append(self._format_after_grace_body())
        result.append(('spanners', bundle.closing.spanners))
        result.append(('commands', bundle.closing.commands))
        result.append(('indicators', bundle.closing.indicators))
        result.append(('comments', bundle.closing.comments))
        return result

    def _format_contents_slot(self, bundle):
        result = []
        result.append(self._format_leaf_body(bundle))
        return result

    def _format_grace_body(self):
        result = []
        if self._grace is not None:
            grace = self._grace
            if len(grace):
                result.append(format(grace))
        return ['grace body', result]

    def _format_leaf_body(self, bundle):
        from abjad.tools import systemtools
        indent = systemtools.LilyPondFormatManager.indent
        result = self._format_leaf_nucleus()[1]
        result.extend(bundle.right.stem_tremolos)
        result.extend(bundle.right.articulations)
        result.extend(bundle.right.commands)
        result.extend(bundle.right.indicators)
        result.extend(bundle.right.spanners)
        result.extend(bundle.right.spanner_stops)
        result.extend(bundle.right.spanner_starts)
        result.extend(bundle.right.comments)
        result = [' '.join(result)]
        markup = bundle.right.markup
        if markup:
            if len(markup) == 1:
                result[0] += ' {}'.format(markup[0])
            else:
                result.extend(indent + '{}'.format(x) for x in markup)
        trill_pitches = bundle.right.trill_pitches
        if trill_pitches:
            assert len(trill_pitches) == 1
            result[-1] += ' {}'.format(trill_pitches[0])
        return ['self body', result]

    def _format_leaf_nucleus(self):
        return ['nucleus', self._body]

    def _format_open_brackets_slot(self, bundle):
        return []

    def _format_opening_slot(self, bundle):
        result = []
        result.append(('comments', bundle.opening.comments))
        result.append(('indicators', bundle.opening.indicators))
        result.append(('commands', bundle.opening.commands))
        result.append(self._format_after_grace_opening())
        result.append(('spanners', bundle.opening.spanners))
        return result

    def _get_format_pieces(self):
        return self._get_lilypond_format().split('\n')

    def _get_format_specification(self):
        summary = self._compact_representation
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            repr_args_values=[summary],
            storage_format_args_values=[format(self, 'lilypond')],
            storage_format_is_indented=False,
            storage_format_kwargs_names=[],
            )

    def _get_leaf(self, n=0):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        Selection = selectiontools.Selection

        def next(component):
            new_component = component._get_nth_component_in_time_order_from(1)
            if new_component is None:
                return
            candidates = new_component._get_descendants_starting_with()
            candidates = [
                x for x in candidates if isinstance(x, scoretools.Leaf)
                ]
            for candidate in candidates:
                if Selection._all_are_components_in_same_logical_voice(
                    [component, candidate]):
                    return candidate

        def previous(component):
            new_component = component._get_nth_component_in_time_order_from(-1)
            if new_component is None:
                return
            candidates = new_component._get_descendants_stopping_with()
            candidates = [
                x for x in candidates if isinstance(x, scoretools.Leaf)
                ]
            for candidate in candidates:
                if Selection._all_are_components_in_same_logical_voice(
                    [component, candidate]):
                    return candidate

        current_leaf = self
        if n < 0:
            for i in range(abs(n)):
                current_leaf = previous(current_leaf)
                if current_leaf is None:
                    break
        elif n == 0:
            pass
        else:
            for i in range(n):
                current_leaf = next(current_leaf)
                if current_leaf is None:
                    break
        return current_leaf

    def _get_leaf_index(self):
        self._update_now(offsets=True)
        return self._leaf_index

    def _get_logical_tie(self):
        from abjad.tools import selectiontools
        from abjad.tools import spannertools
        prototype = (spannertools.Tie,)
        for component in self._get_parentage():
            tie_spanners = component._get_spanners(prototype)
            if len(tie_spanners) == 1:
                tie_spanner = tie_spanners.pop()
                return selectiontools.LogicalTie(
                    music=tie_spanner._get_leaves()
                    )
            elif 1 < len(tie_spanners):
                message = 'parentage of {!r} contains {} tie spanners.'
                message = message.format(self, len(tie_spanners))
                raise Exception(message)
        else:
            return selectiontools.LogicalTie(music=self)

    def _process_contribution_packet(self, contribution_packet):
        manager = systemtools.LilyPondFormatManager
        indent = manager.indent
        result = ''
        for contributor, contributions in contribution_packet:
            if contributions:
                if isinstance(contributor, tuple):
                    contributor = indent + contributor[0] + ':\n'
                else:
                    contributor = indent + contributor + ':\n'
                result += contributor
                for contribution in contributions:
                    contribution = (indent * 2) + contribution + '\n'
                    result += contribution
        return result

    def _report_format_contributors(self):
        manager = systemtools.LilyPondFormatManager
        indent = manager.indent
        bundle = manager.bundle_format_contributions(self)
        report = ''
        report += 'slot 1:\n'
        packet = self._format_before_slot(bundle)
        report += self._process_contribution_packet(packet)
        report += 'slot 3:\n'
        packet = self._format_opening_slot(bundle)
        report += self._process_contribution_packet(packet)
        report += 'slot 4:\n'
        report += indent + 'leaf body:\n'
        string = self._format_contents_slot(bundle)[0][1][0]
        report += (indent * 2) + string + '\n'
        report += 'slot 5:\n'
        packet = self._format_closing_slot(bundle)
        report += self._process_contribution_packet(packet)
        report += 'slot 7:\n'
        packet = self._format_after_slot(bundle)
        report += self._process_contribution_packet(packet)
        while report[-1] == '\n':
            report = report[:-1]
        return report

    def _scale(self, multiplier):
        new_duration = multiplier * self._get_duration()
        self._set_duration(new_duration)

    def _set_duration(self, new_duration, use_messiaen_style_ties=False):
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        new_duration = durationtools.Duration(new_duration)
        # change LilyPond multiplier if leaf already has LilyPond multiplier
        if self._get_indicators(durationtools.Multiplier):
            detach(durationtools.Multiplier, self)
            multiplier = new_duration.__div__(self.written_duration)
            attach(multiplier, self)
            return [self]
        # change written duration if new duration is assignable
        try:
            self.written_duration = new_duration
            return [self]
        except AssignabilityError:
            pass
        # make new notes or tuplets if new duration is nonassignable
        components = scoretools.make_notes(
            0,
            new_duration,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
        if isinstance(components[0], scoretools.Leaf):
            tied_leaf_count = len(components) - 1
            tied_leaves = tied_leaf_count * self
            all_leaves = [self] + tied_leaves
            for x, component in zip(all_leaves, components):
                x.written_duration = component.written_duration
            self._splice(tied_leaves, grow_spanners=True)
            parentage = self._get_parentage()
            if not parentage._get_spanners(spannertools.Tie):
                #if spannertools.Tie._attachment_test(self):
                tie = spannertools.Tie()
                if tie._attachment_test(self):
                    tie = spannertools.Tie(
                        use_messiaen_style_ties=use_messiaen_style_ties,
                        )
                    attach(tie, all_leaves)
            return all_leaves
        else:
            assert isinstance(components[0], scoretools.Tuplet)
            tuplet = components[0]
            components = tuplet[:]
            tied_leaf_count = len(components) - 1
            tied_leaves = tied_leaf_count * self
            all_leaves = [self] + tied_leaves
            for x, component in zip(all_leaves, components):
                x.written_duration = component.written_duration
            self._splice(tied_leaves, grow_spanners=True)
            if not self._get_spanners(spannertools.Tie):
                #if spannertools.Tie._attachment_test(self):
                tie = spannertools.Tie()
                if tie._attachment_test(self):
                    tie = spannertools.Tie(
                        use_messiaen_style_ties=use_messiaen_style_ties,
                        )
                    attach(tie, all_leaves)
            tuplet_multiplier = tuplet.multiplier
            scoretools.Tuplet(tuplet_multiplier, all_leaves)
            return [tuplet]

    def _split(
        self,
        durations,
        cyclic=False,
        fracture_spanners=False,
        tie_split_notes=True,
        use_messiaen_style_ties=False,
        ):
        from abjad.tools import pitchtools
        from abjad.tools import selectiontools
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        durations = [durationtools.Duration(x) for x in durations]
        if cyclic:
            durations = sequencetools.Sequence(durations)
            durations = durations.repeat_to_weight(self._get_duration())
        durations = [durationtools.Duration(x) for x in durations]
        if sum(durations) < self._get_duration():
            last_duration = self._get_duration() - sum(durations)
            durations.append(last_duration)
        weight = self._get_duration()
        durations = sequencetools.Sequence(durations).truncate(weight=weight)
        result = []
        leaf_prolation = self._get_parentage(include_self=False).prolation
        timespan = self._get_timespan()
        start_offset = timespan.start_offset
        for duration in durations:
            new_leaf = copy.copy(self)
            preprolated_duration = duration / leaf_prolation
            shard = new_leaf._set_duration(
                preprolated_duration,
                use_messiaen_style_ties=use_messiaen_style_ties,
                )
            for x in shard:
                if isinstance(x, scoretools.Leaf):
                    x_duration = x.written_duration * leaf_prolation
                else:
                    x_duration = x.multiplied_duration * leaf_prolation
                stop_offset = x_duration + start_offset
                x._start_offset = start_offset
                x._stop_offset = stop_offset
                x._timespan = timespantools.Timespan(
                    start_offset=start_offset,
                    stop_offset=stop_offset,
                    )
                start_offset = stop_offset
            shard = [x._get_parentage().root for x in shard]
            result.append(shard)
        flattened_result = sequencetools.Sequence(result).flatten()
        flattened_result = selectiontools.Selection(flattened_result)
        prototype = (spannertools.Tie,)
        parentage = self._get_parentage()
        if parentage._get_spanners(prototype=prototype):
            selection = select(flattened_result)
            for component in selection:
                # TODO: make top-level detach() work here
                for spanner in component._get_spanners(prototype):
                    spanner._sever_all_components()
                #detach(prototype, component)
        # replace leaf with flattened result
        selection = selectiontools.Selection(self)
        parent, start, stop = selection._get_parent_and_start_stop_indices()
        if parent:
            parent.__setitem__(slice(start, stop + 1), flattened_result)
        else:
            selection._give_dominant_spanners(flattened_result)
            selection._withdraw_from_crossing_spanners()
        # fracture spanners
        if fracture_spanners:
            first_shard = result[0]
            for spanner in first_shard[-1]._get_spanners():
                index = spanner._index(first_shard[-1])
                spanner._fracture(index, direction=Right)
            last_shard = result[-1]
            for spanner in last_shard[0]._get_spanners():
                index = spanner._index(last_shard[0])
                spanner._fracture(index, direction=Left)
            for middle_shard in result[1:-1]:
                for spanner in middle_shard[0]._get_spanners():
                    index = spanner._index(middle_shard[0])
                    spanner._fracture(index, direction=Left)
                for spanner in middle_shard[-1]._get_spanners():
                    index = spanner._index(middle_shard[-1])
                    spanner._fracture(index, direction=Right)
        # adjust first leaf
        self._detach_grace_containers(kind='after')
        # adjust any middle leaves
        for middle_leaf in flattened_result[1:-1]:
            middle_leaf._detach_grace_containers(kind='grace')
            self._detach_grace_containers(kind='after')
            detach(object, middle_leaf)
        # adjust last leaf
        last_leaf = flattened_result[-1]
        last_leaf._detach_grace_containers(kind='grace')
        detach(object, last_leaf)
        # tie split notes, rests and chords as specified
        if pitchtools.Pitch.is_pitch_carrier(self) and tie_split_notes:
            flattened_result_leaves = iterate(flattened_result).by_leaf()
            # TODO: implement Selection._attach_tie_spanner_to_leaves()
            pairs = sequencetools.Sequence(flattened_result_leaves).nwise()
            for leaf_pair in pairs:
                selection = selectiontools.Selection(leaf_pair)
                selection._attach_tie_spanner_to_leaf_pair(
                    use_messiaen_style_ties=use_messiaen_style_ties,
                    )
        # return result
        return result

    # TODO: This should be replaced in favor of self._split().
    #       The precondition is that self._split() must be
    #       extended to handle graces.
    def _split_by_duration(
        self,
        duration,
        fracture_spanners=False,
        tie_split_notes=True,
        use_messiaen_style_ties=False,
        ):
        from abjad.tools import pitchtools
        from abjad.tools import selectiontools
        # check input
        duration = durationtools.Duration(duration)
        # calculate durations
        leaf_multiplied_duration = self._multiplied_duration
        prolation = self._get_parentage(include_self=False).prolation
        preprolated_duration = duration / prolation
        # handle boundary cases
        if preprolated_duration <= 0:
            return ([], [self])
        if leaf_multiplied_duration <= preprolated_duration:
            return ([self], [])
        # create new leaf
        new_leaf = copy.copy(self)
        self._splice([new_leaf], grow_spanners=True)
        # adjust leaf
        self._detach_grace_containers(kind='after')
        # adjust new leaf
        new_leaf._detach_grace_containers(kind='grace')
        left_leaf_list = self._set_duration(
            preprolated_duration,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
        right_preprolated_duration = \
            leaf_multiplied_duration - preprolated_duration
        right_leaf_list = new_leaf._set_duration(
            right_preprolated_duration,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
        leaf_left_of_split = left_leaf_list[-1]
        leaf_right_of_split = right_leaf_list[0]
        leaves_around_split = (leaf_left_of_split, leaf_right_of_split)
        if fracture_spanners:
            for spanner in leaf_left_of_split._get_spanners():
                index = spanner._index(leaf_left_of_split)
                spanner._fracture(index, direction=Right)
        # tie split notes, rests and chords as specified
        if pitchtools.Pitch.is_pitch_carrier(self) and tie_split_notes:
            selection = selectiontools.Selection(leaves_around_split)
            selection._attach_tie_spanner_to_leaf_pair(
                use_messiaen_style_ties=use_messiaen_style_ties,
                )
        return left_leaf_list, right_leaf_list
        # TODO: make this substitution work
        #return self._split(
        #    leaf,
        #    [duration],
        #    cyclic=False,
        #    fracture_spanners=fracture_spanners,
        #    tie_split_notes=tie_split_notes,
        #    )

    def _to_tuplet_with_ratio(self, proportions, is_diminution=True):
        from abjad.tools import scoretools
        # check input
        proportions = mathtools.Ratio(proportions)
        # find target duration of fixed-duration tuplet
        target_duration = self.written_duration
        # find basic duration of note in tuplet
        basic_prolated_duration = target_duration / sum(proportions.numbers)
        # find basic written duration of note in tuplet
        basic_written_duration = \
            basic_prolated_duration.equal_or_greater_assignable
        # find written duration of each note in tuplet
        written_durations = [
            _ * basic_written_duration for _ in proportions.numbers
            ]
        # make tuplet notes
        try:
            notes = [scoretools.Note(0, x) for x in written_durations]
        except AssignabilityError:
            denominator = target_duration.denominator
            note_durations = [
                durationtools.Duration(_, denominator)
                for _ in proportions.numbers
                ]
            notes = scoretools.make_notes(0, note_durations)
        # make tuplet
        tuplet = scoretools.FixedDurationTuplet(target_duration, notes)
        # fix tuplet contents if necessary
        tuplet._fix()
        # change prolation if necessary
        if not tuplet.multiplier == 1:
            if is_diminution:
                if not tuplet.is_diminution:
                    tuplet.toggle_prolation()
            else:
                if tuplet.is_diminution:
                    tuplet.toggle_prolation()
        # return tuplet
        return tuplet

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        return '({})'.format(self._formatted_duration)

    @property
    def _duration_in_seconds(self):
        from abjad.tools import indicatortools
        tempo = self._get_effective(indicatortools.Tempo)
        if tempo is not None and not tempo.is_imprecise:
            result = (
                self._get_duration() /
                tempo.reference_duration /
                tempo.units_per_minute * 60
                )
            return durationtools.Duration(result)
        raise MissingTempoError

    @property
    def _formatted_duration(self):
        duration_string = self.written_duration.lilypond_duration_string
        multiplier = None
        multiplier_prototype = (
            durationtools.Multiplier,
            mathtools.NonreducedFraction,
            )
        multipliers = self._get_indicators(multiplier_prototype)
        if not multipliers:
            pass
        elif len(multipliers) == 1:
            multiplier = multipliers[0]
        elif 1 < len(multipliers):
            message = 'more than one LilyPond duration multiplier.'
            raise ValueError(message)
        if multiplier is not None:
            result = '{} * {!s}'.format(duration_string, multiplier)
        else:
            result = duration_string
        return result

    @property
    def _multiplied_duration(self):
        if self.written_duration:
            multiplier_prototype = (
                durationtools.Multiplier,
                mathtools.NonreducedFraction,
                )
            if self._get_indicators(multiplier_prototype):
                multipliers = self._get_indicators(multiplier_prototype)
                if 1 == len(multipliers):
                    multiplier = multipliers[0]
                    multiplier = durationtools.Duration(multiplier)
                elif 1 < len(multipliers):
                    message = 'more than one duration multiplier.'
                    raise ValueError(message)
                multiplied_duration = multiplier * self.written_duration
                return multiplied_duration
            else:
                return durationtools.Duration(self.written_duration)
        else:
            return None

    @property
    def _preprolated_duration(self):
        return self._multiplied_duration

    ### PUBLIC PROPERTIES ###

    @property
    def written_duration(self):
        '''Written duration of leaf.

        Set to duration.

        Returns duration.
        '''
        return self._written_duration

    @written_duration.setter
    def written_duration(self, argument):
        rational = durationtools.Duration(argument)
        if not rational.is_assignable:
            message = 'not assignable duration: {!r}.'
            message = message.format(rational)
            raise AssignabilityError(message)
        self._written_duration = rational



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_PitchSet___slots___01():
    r'''Named pitch sets are immutable.
    '''

    pitch_classes = ['bf', 'bqf', "fs'", "g'", 'bqf', "g'"]
    pitch_set = pitchtools.PitchSet(pitch_classes)

    assert pytest.raises(AttributeError, "pitch_set.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitch___repr___01():
    r'''Named pitch repr is evaluable.
    '''

    named_pitch_1 = NamedPitch("cs''")
    named_pitch_2 = eval(repr(named_pitch_1))

    r'''
    NamedPitch("cs''")
    '''

    assert isinstance(named_pitch_1, NamedPitch)
    assert isinstance(named_pitch_2, NamedPitch)


def test_pitchtools_NamedPitch___repr___02():
    r'''Repr values.
    '''

    named_pitch = NamedPitch("cs''")

    assert repr(named_pitch) == 'NamedPitch("cs\'\'")'
    assert format(named_pitch, 'storage') == 'pitchtools.NamedPitch("cs\'\'")'



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection__get_component_01():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    assert select(staff)._get_component(Measure, 0) is staff[0]
    assert select(staff)._get_component(Measure, 1) is staff[1]
    assert select(staff)._get_component(Measure, 2) is staff[2]


def test_selectiontools_Selection__get_component_02():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    assert select(staff)._get_component(Measure, -1) is staff[2]
    assert select(staff)._get_component(Measure, -2) is staff[1]
    assert select(staff)._get_component(Measure, -3) is staff[0]


def test_selectiontools_Selection__get_component_03():
    r'''Read forwards for positive n.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    r'''
    \new Staff {
      {
            \time 2/8
            c'8
            d'8
      }
      {
            \time 2/8
            e'8
            f'8
      }
      {
            \time 2/8
            g'8
            a'8
      }
    }
    '''

    assert select(staff)._get_component(scoretools.Leaf, 0) is staff[0][0]
    assert select(staff)._get_component(scoretools.Leaf, 1) is staff[0][1]
    assert select(staff)._get_component(scoretools.Leaf, 2) is staff[1][0]
    assert select(staff)._get_component(scoretools.Leaf, 3) is staff[1][1]
    assert select(staff)._get_component(scoretools.Leaf, 4) is staff[2][0]
    assert select(staff)._get_component(scoretools.Leaf, 5) is staff[2][1]


def test_selectiontools_Selection__get_component_04():
    r'''Read backwards for negative n.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    r'''
    \new Staff {
      {
            \time 2/8
            c'8
            d'8
      }
      {
            \time 2/8
            e'8
            f'8
      }
      {
            \time 2/8
            g'8
            a'8
      }
    }
    '''

    assert select(staff)._get_component(scoretools.Leaf, -1) is staff[2][1]
    assert select(staff)._get_component(scoretools.Leaf, -2) is staff[2][0]
    assert select(staff)._get_component(scoretools.Leaf, -3) is staff[1][1]
    assert select(staff)._get_component(scoretools.Leaf, -4) is staff[1][0]
    assert select(staff)._get_component(scoretools.Leaf, -5) is staff[0][1]
    assert select(staff)._get_component(scoretools.Leaf, -6) is staff[0][0]


def test_selectiontools_Selection__get_component_05():

    staff = Staff(r'''
        c'16
        r16
        d'8
        r8
        e'8.
        r8.
        f'4
        r4
        ''')

    notes = [staff[0], staff[2], staff[4], staff[6]]
    rests = [staff[1], staff[3], staff[5], staff[7]]

    assert select(staff)._get_component(Note, 0) is notes[0]
    assert select(staff)._get_component(Note, 1) is notes[1]
    assert select(staff)._get_component(Note, 2) is notes[2]
    assert select(staff)._get_component(Note, 3) is notes[3]

    assert select(staff)._get_component(Rest, 0) is rests[0]
    assert select(staff)._get_component(Rest, 1) is rests[1]
    assert select(staff)._get_component(Rest, 2) is rests[2]
    assert select(staff)._get_component(Rest, 3) is rests[3]

    assert select(staff)._get_component(Staff, 0) is staff


def test_selectiontools_Selection__get_component_06():
    r'''Iterates backwards with negative values of n.
    '''

    staff = Staff(r'''
        c'16
        r16
        d'8
        r8
        e'8.
        r8.
        f'4
        r4
        ''')

    notes = [staff[0], staff[2], staff[4], staff[6]]
    rests = [staff[1], staff[3], staff[5], staff[7]]

    assert select(staff)._get_component(Note, -1) is notes[3]
    assert select(staff)._get_component(Note, -2) is notes[2]
    assert select(staff)._get_component(Note, -3) is notes[1]
    assert select(staff)._get_component(Note, -4) is notes[0]

    assert select(staff)._get_component(Rest, -1) is rests[3]
    assert select(staff)._get_component(Rest, -2) is rests[2]
    assert select(staff)._get_component(Rest, -3) is rests[1]
    assert select(staff)._get_component(Rest, -4) is rests[0]



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_UnweightedSearchTree___call___01():

    definition = {
            2: {
                2: {
                    2: None
                },
                3: None
            },
            5: None
        }
    search_tree = quantizationtools.UnweightedSearchTree(definition)
    q_grid = quantizationtools.QGrid()
    a = quantizationtools.QEventProxy(quantizationtools.SilentQEvent(0,      ['A'], index=1), 0, 1)
    b = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 5), ['B'], index=2), 0, 1)
    c = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 4), ['C'], index=3), 0, 1)
    d = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 3), ['D'], index=4), 0, 1)
    e = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 5), ['E'], index=5), 0, 1)
    f = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 2), ['F'], index=6), 0, 1)
    g = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 5), ['G'], index=7), 0, 1)
    h = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 3), ['H'], index=8), 0, 1)
    i = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 4), ['I'], index=9), 0, 1)
    j = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((4, 5), ['J'], index=10), 0, 1)
    k = quantizationtools.QEventProxy(quantizationtools.SilentQEvent(1,      ['K'], index=11), 0, 1)
    q_grid.fit_q_events([a, b, c, d, e, f, g, h, i, j, k])

    q_grids = search_tree(q_grid)

    assert q_grids[0].root_node.rtm_format == '(1 (1 1))'
    assert q_grids[1].root_node.rtm_format == '(1 (1 1 1 1 1))'



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Tuplet_grob_override_01():
    r'''Tuplets bracket grob overrides at before and after slots.
    '''

    tuplet = Tuplet((2, 3), "c'8 d'8 e'8 f'8")
    override(tuplet).glissando.thickness = 3

    assert format(tuplet) == stringtools.normalize(
        r'''
        \override Glissando.thickness = #3
        \tweak edge-height #'(0.7 . 0)
        \times 2/3 {
            c'8
            d'8
            e'8
            f'8
        }
        \revert Glissando.thickness
        '''
        )



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_InspectionAgent_get_duration_01():
    r'''Spanner duration in seconds equals sum of duration
    of all leaves in spanner, in seconds.
    '''

    voice = Voice([
        Measure((2, 12), "c'8 d'8", implicit_scaling=True),
        Measure((2, 8), "c'8 d'8")]
        )
    leaves = select(voice).by_leaf()
    tempo = Tempo(Duration(1, 8), 42)
    attach(tempo, voice, scope=Voice)
    beam = Beam()
    attach(beam, leaves)
    crescendo = Crescendo()
    attach(crescendo, voice[0][:])
    decrescendo = Decrescendo()
    attach(decrescendo, voice[1][:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \tempo 8=42
            {
                \time 2/12
                \scaleDurations #'(2 . 3) {
                    c'8 [ \<
                    d'8 \!
                }
            }
            {
                \time 2/8
                c'8 \>
                d'8 ] \!
            }
        }
        '''
        )

    assert inspect_(beam).get_duration(in_seconds=True) == Duration(100, 21)
    assert inspect_(crescendo).get_duration(in_seconds=True) == Duration(40, 21)
    assert inspect_(decrescendo).get_duration(in_seconds=True) == \
        Duration(20, 7)


def test_agenttools_InspectionAgent_get_duration_02():

    voice = Voice(
        [Measure((2, 12), "c'8 d'8", implicit_scaling=True),
        Measure((2, 8), "c'8 d'8")]
        )
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    crescendo = Crescendo()
    attach(crescendo, voice[0][:])
    decrescendo = Decrescendo()
    attach(decrescendo, voice[1][:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/12
                \scaleDurations #'(2 . 3) {
                    c'8 [ \<
                    d'8 \!
                }
            }
            {
                \time 2/8
                c'8 \>
                d'8 ] \!
            }
        }
        '''
        )

    assert inspect_(beam).get_duration() == Duration(5, 12)
    assert inspect_(crescendo).get_duration() == Duration(2, 12)
    assert inspect_(decrescendo).get_duration() == Duration(2, 8)


def test_agenttools_InspectionAgent_get_duration_03():
    r'''Container duration in seconds equals
    sum of leaf durations in seconds.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    tempo = Tempo(Duration(1, 4), 38)
    attach(tempo, staff)
    tempo = Tempo(Duration(1, 4), 42)
    attach(tempo, staff[2])
    score = Score([staff])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tempo 4=38
                c'8
                d'8
                \tempo 4=42
                e'8
                f'8
            }
        >>
        '''
        )

    assert inspect_(score).get_duration(in_seconds=True) == Duration(400, 133)


def test_agenttools_InspectionAgent_get_duration_04():
    r'''Container can not calculate duration in seconds
    without tempo indication.
    '''

    container = Container("c'8 d'8 e'8 f'8")
    statement = 'inspect_(container).get_duration(in_seconds=True)'
    assert pytest.raises(Exception, statement)


def test_agenttools_InspectionAgent_get_duration_05():
    r'''Clock duration equals duration divide by effective tempo.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    tempo = Tempo(Duration(1, 4), 38)
    attach(tempo, staff)
    tempo = Tempo(Duration(1, 4), 42)
    attach(tempo, staff[2])
    Score([staff])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tempo 4=38
            c'8
            d'8
            \tempo 4=42
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff[0]).get_duration(in_seconds=True) == Duration(15, 19)
    assert inspect_(staff[1]).get_duration(in_seconds=True) == Duration(15, 19)
    assert inspect_(staff[2]).get_duration(in_seconds=True) == Duration(5, 7)
    assert inspect_(staff[3]).get_duration(in_seconds=True) == Duration(5, 7)


def test_agenttools_InspectionAgent_get_duration_06():
    r'''Clock duration can not calculate without tempo.
    '''

    note = Note("c'4")
    statement = 'inspect_(note).get_duration(in_seconds=True)'
    assert pytest.raises(Exception, statement)
